import argparse
import numpy as np
import os
import tokenization
import sys
import json
import collections
import logging
import six


class SquadExample(object):
  """A single training/test example for simple sequence classification.

     For examples without an answer, the start and end position are -1.
  """

  def __init__(self,
               qas_id,
               question_text,
               doc_tokens,
               orig_answer_text=None,
               start_position=None,
               end_position=None,
               is_impossible=False):
    self.qas_id = qas_id
    self.question_text = question_text
    self.doc_tokens = doc_tokens
    self.orig_answer_text = orig_answer_text
    self.start_position = start_position
    self.end_position = end_position
    self.is_impossible = is_impossible

  def __str__(self):
    return self.__repr__()

  def __repr__(self):
    s = ""
    s += "qas_id: %s" % (tokenization.printable_text(self.qas_id))
    s += ", question_text: %s" % (
        tokenization.printable_text(self.question_text))
    s += ", doc_tokens: [%s]" % (" ".join(self.doc_tokens))
    if self.start_position:
      s += ", start_position: %d" % (self.start_position)
    if self.start_position:
      s += ", end_position: %d" % (self.end_position)
    if self.start_position:
      s += ", is_impossible: %r" % (self.is_impossible)
    return s


class InputFeatures(object):
  """A single set of features of data."""

  def __init__(self,
               unique_id,
               example_index,
               doc_span_index,
               tokens,
               token_to_orig_map,
               token_is_max_context,
               input_ids,
               input_mask,
               segment_ids,
               start_position=None,
               end_position=None,
               is_impossible=None):
    self.unique_id = unique_id
    self.example_index = example_index
    self.doc_span_index = doc_span_index
    self.tokens = tokens
    self.token_to_orig_map = token_to_orig_map
    self.token_is_max_context = token_is_max_context
    self.input_ids = input_ids
    self.input_mask = input_mask
    self.segment_ids = segment_ids
    self.start_position = start_position
    self.end_position = end_position
    self.is_impossible = is_impossible

def read_squad_examples(input_file, is_training, version_2_with_negative):
  """Read a SQuAD json file into a list of SquadExample."""
  with open(input_file, "r", encoding='utf-8') as reader:
    input_data = json.load(reader)["data"]

  def is_whitespace(c):
    if c == " " or c == "\t" or c == "\r" or c == "\n" or ord(c) == 0x202F:
      return True
    return False

  examples = []
  for entry in input_data:
    for paragraph in entry["paragraphs"]:
      paragraph_text = paragraph["context"]
      doc_tokens = []
      char_to_word_offset = []
      prev_is_whitespace = True
      for c in paragraph_text:
        if is_whitespace(c):
          prev_is_whitespace = True
        else:
          if prev_is_whitespace:
            doc_tokens.append(c)
          else:
            doc_tokens[-1] += c
          prev_is_whitespace = False
        char_to_word_offset.append(len(doc_tokens) - 1)

      for qa in paragraph["qas"]:
        qas_id = qa["id"]
        question_text = qa["question"]
        start_position = None
        end_position = None
        orig_answer_text = None
        is_impossible = False
        if is_training:
          if version_2_with_negative:
            is_impossible = qa["is_impossible"]
          if (len(qa["answers"]) != 1) and (not is_impossible):
            raise ValueError(
                "For training, each question should have exactly 1 answer.")
          if not is_impossible:
            answer = qa["answers"][0]
            orig_answer_text = answer["text"]
            answer_offset = answer["answer_start"]
            answer_length = len(orig_answer_text)
            start_position = char_to_word_offset[answer_offset]
            end_position = char_to_word_offset[answer_offset + answer_length -
                                               1]
            # Only add answers where the text can be exactly recovered from the
            # document. If this CAN'T happen it's likely due to weird Unicode
            # stuff so we will just skip the example.
            #
            # Note that this means for training mode, every example is NOT
            # guaranteed to be preserved.
            actual_text = " ".join(
                doc_tokens[start_position:(end_position + 1)])
            cleaned_answer_text = " ".join(
                tokenization.whitespace_tokenize(orig_answer_text))
            if actual_text.find(cleaned_answer_text) == -1:
              logging.warning("Could not find answer: '%s' vs. '%s'",
                                 actual_text, cleaned_answer_text)
              continue
          else:
            start_position = -1
            end_position = -1
            orig_answer_text = ""

        example = SquadExample(
            qas_id=qas_id,
            question_text=question_text,
            doc_tokens=doc_tokens,
            orig_answer_text=orig_answer_text,
            start_position=start_position,
            end_position=end_position,
            is_impossible=is_impossible)
        examples.append(example)

  return examples

def batch_and_extract_features(features, batch_size, max_sequence_length):
    """ Concat the input according to the batch_size """

    def chunks(l, n):
        for i in range(0, len(l), n):
            yield l[i:i + n]

    batch_input_ids = []
    batch_input_mask = []
    batch_segment_ids = []
    feature_chunks = list(chunks(features, batch_size))

    for feature_chunk in feature_chunks:
        input_ids = [
            np.expand_dims(feature.input_ids, 0) for feature in feature_chunk
        ]
        input_mask = [
            np.expand_dims(feature.input_mask, 0) for feature in feature_chunk
        ]
        segment_ids = [
            np.expand_dims(feature.segment_ids, 0) for feature in feature_chunk
        ]
        input_ids = np.concatenate(input_ids)
        input_mask = np.concatenate(input_mask)
        segment_ids = np.concatenate(segment_ids)
        batch_input_ids.append(input_ids)
        batch_input_mask.append(input_mask)
        batch_segment_ids.append(segment_ids)

    return batch_input_ids, batch_input_mask, batch_segment_ids

def _check_is_max_context(doc_spans, cur_span_index, position):
  """Check if this is the 'max context' doc span for the token."""

  # Because of the sliding window approach taken to scoring documents, a single
  # token can appear in multiple documents. E.g.
  #  Doc: the man went to the store and bought a gallon of milk
  #  Span A: the man went to the
  #  Span B: to the store and bought
  #  Span C: and bought a gallon of
  #  ...
  #
  # Now the word 'bought' will have two scores from spans B and C. We only
  # want to consider the score with "maximum context", which we define as
  # the *minimum* of its left and right context (the *sum* of left and
  # right context will always be the same, of course).
  #
  # In the example the maximum context for 'bought' would be span C since
  # it has 1 left context and 3 right context, while span B has 4 left context
  # and 0 right context.
  best_score = None
  best_span_index = None
  for (span_index, doc_span) in enumerate(doc_spans):
    end = doc_span.start + doc_span.length - 1
    if position < doc_span.start:
      continue
    if position > end:
      continue
    num_left_context = position - doc_span.start
    num_right_context = end - position
    score = min(num_left_context, num_right_context) + 0.01 * doc_span.length
    if best_score is None or score > best_score:
      best_score = score
      best_span_index = span_index

  return cur_span_index == best_span_index

def convert_examples_to_features(examples, tokenizer, max_seq_length,
                                 doc_stride, max_query_length, is_training):
  """Loads a data file into a list of `InputBatch`s."""

  unique_id = 1000000000

  features = []
  for (example_index, example) in enumerate(examples):
    query_tokens = tokenizer.tokenize(example.question_text)

    if len(query_tokens) > max_query_length:
      query_tokens = query_tokens[0:max_query_length]

    tok_to_orig_index = []
    orig_to_tok_index = []
    all_doc_tokens = []
    for (i, token) in enumerate(example.doc_tokens):
      orig_to_tok_index.append(len(all_doc_tokens))
      sub_tokens = tokenizer.tokenize(token)
      for sub_token in sub_tokens:
        tok_to_orig_index.append(i)
        all_doc_tokens.append(sub_token)

    tok_start_position = None
    tok_end_position = None
    if is_training and example.is_impossible:
      tok_start_position = -1
      tok_end_position = -1
    if is_training and not example.is_impossible:
      tok_start_position = orig_to_tok_index[example.start_position]
      if example.end_position < len(example.doc_tokens) - 1:
        tok_end_position = orig_to_tok_index[example.end_position + 1] - 1
      else:
        tok_end_position = len(all_doc_tokens) - 1
      (tok_start_position, tok_end_position) = _improve_answer_span(
          all_doc_tokens, tok_start_position, tok_end_position, tokenizer,
          example.orig_answer_text)

    # The -3 accounts for [CLS], [SEP] and [SEP]
    max_tokens_for_doc = max_seq_length - len(query_tokens) - 3

    # We can have documents that are longer than the maximum sequence length.
    # To deal with this we do a sliding window approach, where we take chunks
    # of the up to our max length with a stride of `doc_stride`.
    _DocSpan = collections.namedtuple(  # pylint: disable=invalid-name
        "DocSpan", ["start", "length"])
    doc_spans = []
    start_offset = 0
    while start_offset < len(all_doc_tokens):
      length = len(all_doc_tokens) - start_offset
      if length > max_tokens_for_doc:
        length = max_tokens_for_doc
      doc_spans.append(_DocSpan(start=start_offset, length=length))
      if start_offset + length == len(all_doc_tokens):
        break
      start_offset += min(length, doc_stride)

    for (doc_span_index, doc_span) in enumerate(doc_spans):
      tokens = []
      token_to_orig_map = {}
      token_is_max_context = {}
      segment_ids = []
      tokens.append("[CLS]")
      segment_ids.append(0)
      for token in query_tokens:
        tokens.append(token)
        segment_ids.append(0)
      tokens.append("[SEP]")
      segment_ids.append(0)

      for i in range(doc_span.length):
        split_token_index = doc_span.start + i
        token_to_orig_map[len(tokens)] = tok_to_orig_index[split_token_index]

        is_max_context = _check_is_max_context(doc_spans, doc_span_index,
                                               split_token_index)
        token_is_max_context[len(tokens)] = is_max_context
        tokens.append(all_doc_tokens[split_token_index])
        segment_ids.append(1)
      tokens.append("[SEP]")
      segment_ids.append(1)

      input_ids = tokenizer.convert_tokens_to_ids(tokens)

      # The mask has 1 for real tokens and 0 for padding tokens. Only real
      # tokens are attended to.
      input_mask = [1] * len(input_ids)

      # Zero-pad up to the sequence length.
      while len(input_ids) < max_seq_length:
        input_ids.append(0)
        input_mask.append(0)
        segment_ids.append(0)

      assert len(input_ids) == max_seq_length
      assert len(input_mask) == max_seq_length
      assert len(segment_ids) == max_seq_length

      start_position = None
      end_position = None
      if is_training and not example.is_impossible:
        # For training, if our document chunk does not contain an annotation
        # we throw it out, since there is nothing to predict.
        doc_start = doc_span.start
        doc_end = doc_span.start + doc_span.length - 1
        out_of_span = False
        if not (tok_start_position >= doc_start and
                tok_end_position <= doc_end):
          out_of_span = True
        if out_of_span:
          start_position = 0
          end_position = 0
        else:
          doc_offset = len(query_tokens) + 2
          start_position = tok_start_position - doc_start + doc_offset
          end_position = tok_end_position - doc_start + doc_offset

      if is_training and example.is_impossible:
        start_position = 0
        end_position = 0

      if example_index < 20:
        logging.info("*** Example ***")
        logging.info("unique_id: %s" % (unique_id))
        logging.info("example_index: %s" % (example_index))
        logging.info("doc_span_index: %s" % (doc_span_index))
        logging.info("tokens: %s" % " ".join(
         [tokenization.printable_text(x) for x in tokens]))
        logging.info("token_to_orig_map: %s" % " ".join(
         ["%d:%d" % (x, y) for (x, y) in six.iteritems(token_to_orig_map)]))
        logging.info("token_is_max_context: %s" % " ".join([
            "%d:%s" % (x, y) for (x, y) in six.iteritems(token_is_max_context)
        ]))
        logging.info("input_ids: %s" % " ".join([str(x) for x in input_ids]))
        logging.info(
         "input_mask: %s" % " ".join([str(x) for x in input_mask]))
        logging.info(
            "segment_ids: %s" % " ".join([str(x) for x in segment_ids]))
        if is_training and example.is_impossible:
          logging.info("impossible example")
        if is_training and not example.is_impossible:
          wer_text = " ".join(tokens[start_position:(end_position + 1)])
          logging.info("start_position: %d" % (start_position))
          logging.info("end_position: %d" % (end_position))
          logging.info(
              "answer: %s" % (tokenization.printable_text(answer_text)))

      feature = InputFeatures(
          unique_id=unique_id,
          example_index=example_index,
          doc_span_index=doc_span_index,
          tokens=tokens,
          token_to_orig_map=token_to_orig_map,
          token_is_max_context=token_is_max_context,
          input_ids=input_ids,
          input_mask=input_mask,
          segment_ids=segment_ids,
          start_position=start_position,
          end_position=end_position,
          is_impossible=example.is_impossible)

      features.append(feature)

      unique_id += 1
  return features

def preprocess(vocab_file, squad_json, batch_size, max_sequence_length,
               max_query_length, doc_stride, version_2_with_negative):
    tokenizer = tokenization.FullTokenizer(vocab_file=vocab_file,
                                           do_lower_case=True)
    eval_examples = read_squad_examples(squad_json,
                                        False,
                                        version_2_with_negative)
    eval_features = convert_examples_to_features(
        examples=eval_examples,
        tokenizer=tokenizer,
        max_seq_length=max_sequence_length,
        doc_stride=doc_stride,
        max_query_length=max_query_length,
        is_training=False)
    input_ids, input_mask, segment_ids = batch_and_extract_features(
        eval_features, batch_size, max_sequence_length)
    return input_ids, input_mask, segment_ids, eval_features, eval_examples




import os 

import torch
import torchvision 

if __name__ == "__main__":
    
    resnet50_model = torchvision.models.resnet50(pretrained=False)
    
    resnet50_model.load_state_dict(torch.load("../data/models/resnet50-0676ba61.pth",map_location = 'cpu'))
    resnet50_model.eval()

    inputs = torch.empty(1,3,224,224)

    torch.jit.save(torch.jit.trace(resnet50_model,inputs),'../data/models/resnet50.pt')
    print("successfully save pt")



import argparse
import magicmind.python.runtime as mm
import torch
import os
from tqdm import tqdm
import numpy as np
from sklearn.metrics import accuracy_score
import transformers
from scipy.special import softmax
import warnings
warnings.filterwarnings("ignore")
from preprocess import preprocess
import sys
from mm_runner import MMRunner
from logger import Logger
from utils import Record
logging = Logger()

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type = int, default = 0, help = "device_id")
parser.add_argument("--magicmind_model", "--magicmind_model", type = str, default = "../data/models/bert_qa_pytorch_model")
parser.add_argument("--batch_size", "--batch_size", type = int, default = 16, help = "batch_size")
parser.add_argument("--max_seq_length", "--max_seq_length", type = int, default = 128, help = "max_seq_length")
parser.add_argument("--compute_accuracy", "--compute_accuracy", type = bool, default = True)
parser.add_argument("--output_dir", "--output_dir", type = str, default = "../data/jsons/output")
parser.add_argument("--acc_result", "--acc_result", type = str, default = "../data/jsons/output/acc_result.txt")


if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind model first!!!")
        exit()
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)

    tokenizer, eval_dataloader = preprocess(args.batch_size, args.max_seq_length)
    all_results = []
    all_labels = []
    epoch_iterator = tqdm(eval_dataloader, desc="Iteration")
    for idx, batch in enumerate(epoch_iterator):
        text_batchs, text_labels = batch['text'], batch['label']
        encoding = tokenizer(text_batchs, return_tensors='np', padding="max_length", truncation=True,max_length=args.max_seq_length)
        inputs = [encoding['input_ids'], encoding['attention_mask'], encoding['token_type_ids']]
        # 处理输出数据
        scores = model(inputs)[0]
        senti_value = np.argmax(scores, axis=1)
        all_labels.extend(text_labels.numpy())
        all_results.extend(senti_value)
    ### 精度计算
    if args.compute_accuracy:

        acc_result = Record(args.acc_result)
        acc = accuracy_score(all_labels,all_results)
        acc_result.write("Accuracy results: {}".format(acc), True)



import numpy as np
from torchvision import transforms
import os
from PIL import Image
import cv2
import logging

def imagenet_dataset(
    val_txt,
    image_file_path,
    count=-1
):
    with open(val_txt, "r") as f:
        lines = f.readlines()
    logging.info("%d pictures will be read." % len(lines))
    current_count = 1
    for line in lines:
        image_name, label = line.split(" ")
        image_path = os.path.join(image_file_path, image_name)
        img = cv2.imread(image_path)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        yield img, label.strip()
        current_count += 1
        if current_count > count and count != -1:
            break

def preprocess(input_image, transpose):
    resize_h, resize_w = (256, 256)
    crop_h, crop_w = (224, 224)
    # resize
    scale = 1.0 * resize_h / min(input_image.shape[0], input_image.shape[1])
    resized = cv2.resize(input_image, (round(scale * input_image.shape[1]), round(scale * input_image.shape[0])))
    # # center crop
    x = resized.shape[1] / 2 - crop_w / 2
    y = resized.shape[0] / 2 - crop_h / 2
    crop_img = resized[int(y):int(y+crop_h), int(x):int(x+crop_w)]
    if transpose:
        input_numpy = np.transpose(crop_img, (1, 2, 0))
    else: 
        input_numpy = crop_img
    return input_numpy



import argparse
import os
from typing import List, Tuple

from magicmind.python.cc._pywrap_runtime import ITensor
from magicmind.python.cc._pywrap_runtime.enums import DataType
from magicmind.python.runtime import (
    Network,
    Dims,
    BuilderConfig,
    Calibrator,
    RemoteConfig,
    QuantizationAlgorithm,
    Builder,
)
from magicmind.python.runtime.model import Model

from logger import Logger

log = Logger()


MODEL_CONFIG_PARAMS = [
    "input_dims",
    "batch_size",
]

# cambricon-note: add dynamic_shape
MODEL_BUILDER_PARAMS = [
    "mlu_arch",
    "precision",
    "activation_quant_algo",
    "weight_quant_granularity",
    "cluster_num",
    "means",
    "vars",
    "toolchain_path",
    "input_layout",
    "output_layout",
    "dim_range_min",
    "dim_range_max",
    "input_dims",
    "dynamic_shape",
    "computation_preference",
    "type64to32_conversion",
    "conv_scale_fold",
    "build_config",
]

MODEL_CALIBRATION_PARAM = [
    "random_calib_range",
    "device_id",
    "file_list",
    "calibration_data_path",
    "rpc_server",
    "calibration_algo",
    "input_layout",
    "rgb2bgr",
]

MODEL_BUILD_AND_SERIALIZE_PARAM = ["input_dtypes", "output_dtypes", "magicmind_model"]


def extract_params(group: str, args: argparse.Namespace) -> dict:
    """Extract spcified parameters from command line parameters.

    Args:
        group(str): One of {"MODEL_CONFIG_PARAMS", "MODEL_BUILDER_PARAMS",
                            "MODEL_CALIBRATION_PARAM", "MODEL_BUILD_AND_SERIALIZE_PARAM"}
        args(argparse.Namespace): In argument, all parameters from command line.

    Returns:
        - Returns filtered arguments.
    """
    assert isinstance(args, argparse.Namespace), "Got a args with invalid type."
    selected_group = None
    global MODEL_CONFIG_PARAMS, MODEL_BUILDER_PARAMS, MODEL_CALIBRATION_PARAM, MODEL_BUILD_AND_SERIALIZE_PARAM
    assert len(group) > 6, "invalid group name"
    if group.upper() in "MODEL_CONFIG_PARAMS":
        selected_group = MODEL_CONFIG_PARAMS
    elif group.upper() in "MODEL_BUILDER_PARAMS":
        selected_group = MODEL_BUILDER_PARAMS
    elif group.upper() in "MODEL_CALIBRATION_PARAM":
        selected_group = MODEL_CALIBRATION_PARAM
    elif group.upper() in "MODEL_BUILD_AND_SERIALIZE_PARAM":
        selected_group = MODEL_BUILD_AND_SERIALIZE_PARAM
    else:
        raise ValueError("unkonwn group name")
    filtered_args = {
        key: getattr(args, key) for key in selected_group if hasattr(args, key)
    }
    return filtered_args


def check_model_config_params_validness(args: dict):
    """Naive check for the validty of each model configuration parameter.

    Args:
        args(dict): A dict with all parameters for model configuration.
    """
    for item in MODEL_CONFIG_PARAMS:
        if item in args.keys():
            if isinstance(args[item], list):
                assert len(args[item]) > 0, "invalid paramter presented in {}".format(
                    item
                )
        else:
            log.info(
                "parameter {} is not specified, maybe use default value.".format(item)
            )


def param_input_dims_to_mm_dims(input_dims: List[List[int]]) -> List[Dims]:
    """Covert input_dims to magicmind Dims format.
    Args:
        input_dims(List[List[int]]]: In parameter, shape of each input.

    Returns:
        -Returns a list with shape of magicmind Dims for each network input.
    """
    assert (
        isinstance(input_dims, list) and len(input_dims) > 0
    ), "invalid parameter input_dims: " + str(input_dims)
    for dim in input_dims:
        assert isinstance(dim, list) and len(dim) > 0, "invalid parameter"
    return [Dims(e) for e in input_dims]


def print_parameters(args: dict, group: list, description: str):
    """Print a group of arguments.

    Args:
        args(dict): A dict includes all parameters.
        group(list): A list of items to be printed.
        description(str): A description for the group of printed parameters.
    """
    log.info("=" * 30)
    log.info(description)
    for item in group:
        if item in args.keys():
            log.info(item + ": " + str(args[item]))
    log.info("=" * 30)


def config_network(net: Network, args: dict) -> bool:
    """Configure input shape, batch size etc. of the network.

    Args:
        net(Network): Network to be configured.

    Returns:
        - Returns Ture if configure successfully.
    """
    check_model_config_params_validness(args)
    print_parameters(
        args, MODEL_CONFIG_PARAMS, "parameters about network configuration:"
    )
    input_count = net.get_input_count()
    # set input dimension to be fixed args["input_dims"] if dim_range is not specified
    if "input_dims" in args.keys() and args["input_dims"] is not None:
        log.info("config_network: going to reset input dims.")
        mm_input_dims = param_input_dims_to_mm_dims(args["input_dims"])
        assert input_count == len(
            mm_input_dims
        ), "config_network: network has {} inputs, but got {} input_dims from parameter.".format(
            input_count, len(mm_input_dims)
        )
        for i in range(input_count):
            tensor = net.get_input(i)
            assert (
                tensor.set_dimension(mm_input_dims[i]).ok() == True
            ), "Failed to set input dimension"
        log.info("config_network: set input dimensions sucessfully.")

    # set batch size
    if "batch_size" in args.keys() and args["batch_size"] is not None:
        batch_size_list = args["batch_size"]
        assert (
            isinstance(batch_size_list, list) and len(batch_size_list) > 0
        ), "invalid paramter batch_size : " + str(batch_size_list)
        assert (
            len(batch_size_list) == input_count
        ), "config_network: network has {} inputs, but got {} batch sizes from parameter.".format(
            input_count, len(batch_size_list)
        )
        log.info("config_network: going to reset input batch size.")
        for i in range(input_count):
            tensor = net.get_input(i)
            dim = tensor.get_dimension().GetDims()
            if len(dim) > 0:
                dim[0] = batch_size_list[i]
                assert tensor.set_dimension(Dims(dim)).ok(), "Failed to set batch size"
            else:
                log.info(
                    "config_network: Input {} is with rank {}, batch size {} will not be applied.".format(
                        i, len(dim), batch_size_list[i]
                    )
                )
        log.info("config_network: set input batch size sucessfully.")

    log.info("config_network: network was configured successfully.")
    return True


def get_mlu_arches_from_param(args: dict) -> list:
    """Get the mlu architectures specified from command line.

    Args:
        args(dict): All arguments.

    Returns:
        - Returns specified mlu architecture or the defalut value (ARCH_CANDIDATES) if
          no one was specified.
    """
    ARCH_CANDIDATES = ["mtp_372", "tp_322", "mtp_592"]
    assert "mlu_arch" in args.keys(), "mlu_arch is required, but didn't specified."
    mlu_arches = None
    if args["mlu_arch"] is None:
        log.info(
            "mlu archtecture has been set to the default value {} since it was not specified.".format(
                ARCH_CANDIDATES
            )
        )
        mlu_arches = ARCH_CANDIDATES
    elif isinstance(args["mlu_arch"], str):
        assert args["mlu_arch"] in ARCH_CANDIDATES, "invalid mlu architecture"
        mlu_arches = [args["mlu_arch"]]
    elif isinstance(args["mlu_arch"], list):
        assert all(
            [e in ARCH_CANDIDATES for e in args["mlu_arch"]]
        ), "invalid mlu architecture"
        mlu_arches = args["mlu_arch"]
    else:
        raise ValueError("invalid parameter")
    return mlu_arches


def get_precision_json_str_from_param(args: dict) -> str:
    """Convert parameter precision to the json str version.

    Args:
        args(dict): All arguments.

    Returns:
        - Returns a json string representing precision.
    """
    PRECISION_CANDIDATES = [
        "force_float32",
        "force_float16",
        "qint16_mixed_float16",
        "qint8_mixed_float16",
        "qint16_mixed_float32",
        "qint8_mixed_float32",
    ]
    assert "precision" in args.keys(), "precision is required, but didn't specified."
    assert args["precision"] in PRECISION_CANDIDATES, "invalid precision"
    json_str = '{"precision_config":{"precision_mode":"' + args["precision"] + '"}}'
    return json_str


def get_default_cluster_num_of_arches(arch_list: List[str]) -> list:
    """get default cluster number for each architecture.

    Args:
        arch_list(dict): Specified architectures list..

    Returns:
        - Returns a list with cluster number for each architecture.
    """
    # note: default cluster num is not the optimal value, is just a valid one.
    DEFAULT_CLUSTER_NUM_4_EACH_ARCH = {
        "mtp_220": [1],
        "mtp_270": [1],
        "mtp_290": [1],
        "mtp_372": [2,6,8],
        "tp_322": [1],
        "mtp_592": [1],
    }
    default_val = [DEFAULT_CLUSTER_NUM_4_EACH_ARCH[e] for e in arch_list]
    log.info(
        "cluster num for arches {} has been set to default value {} repectively "
        "since it was not specified.".format(
            ",".join(arch_list), ",".join([str(e) for e in default_val])
        )
    )
    return default_val


def set_quant_granul_and_algori(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set weight quantization granularity and quantization algorithm

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    # set weight_quant_granularity
    assert (
        "weight_quant_granularity" in args.keys()
        and args["weight_quant_granularity"] is not None
    ), "weight_quant_granularity is not specified."
    weight_quant_granularity = args["weight_quant_granularity"]
    assert weight_quant_granularity in [
        "per_tensor",
        "per_axis",
    ], "invalid weight_quant_granularity: " + str(weight_quant_granularity)
    weight_quant_granul_str = (
        '{"precision_config": {"weight_quant_granularity": "'
        + weight_quant_granularity
        + '"}}'
    )
    assert config.parse_from_string(weight_quant_granul_str).ok()
    log.info("get_build_config: set weight_quant_granularity successfully.")

    # set symmeteric quantization or assymmeteric quantization
    assert (
        "activation_quant_algo" in args.keys() and args["activation_quant_algo"] is not None
    ), "invalid quantization algorithm"
    activation_quant_algo = args["activation_quant_algo"]
    assert activation_quant_algo in [
        "symmetric",
        "asymmetric",
    ], "invalid quantization algorithm: " + str(activation_quant_algo)
    symmetric_qunat_str = (
        '{"precision_config": {"activation_quant_algo":"' + activation_quant_algo + '"}}'
    )
    assert config.parse_from_string(symmetric_qunat_str).ok()
    log.info(
        "get_build_config: set {} quantization successfully.".format(activation_quant_algo)
    )
    return config


def set_cluster_num(
    config: BuilderConfig, mlu_arch_list: List[str], args: dict
) -> BuilderConfig:
    """Set cluster_num to the specified MLU architecture.

    Args:
        config(BuilderConfig): Configuration for builder.
        mlu_arch_list(List[str]): Architectures to be specified cluster_num.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    # set bitmap of visible cluster for each architecture
    # example: "archs": [{"mtp_372": [cluster_num_1, cluster_num_2, cluster_num_3]}]
    cluster_num_list = args["cluster_num"]
    if cluster_num_list is None:
        cluster_num_list = get_default_cluster_num_of_arches(mlu_arch_list)
    assert len(mlu_arch_list) == len(
        cluster_num_list
    ), "{} mlu architectures are specified".format(
        len(mlu_arch_list)
    ) + ", each of which require a cluster_num, but just got {} cluster_num.".format(
        len(cluster_num_list)
    )
    if len(cluster_num_list) > 0 and len(mlu_arch_list) > 0:
        cluster_str = '{"archs": ['
        for i in range(len(mlu_arch_list)):
            cluster_str += '{"' + mlu_arch_list[i] + '": ['
            cluster_str += str(cluster_num_list[i][0])
            for e in range(1, len(cluster_num_list[i])):
                cluster_str += ", " + str(cluster_num_list[i][e])
            cluster_str += "]"
            cluster_str += "}"
        cluster_str += "]}"
    assert config.parse_from_string(cluster_str).ok()
    log.info("get_build_config: set cluster number successfully." + cluster_str)
    return config


def insert_bn_before_first_node(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Insert a batch norm node before the 1st node.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    # an example of mean_var_str:
    # { "insert_bn_before_firstnode": {
    # "0": {"mean": [1.0, 2.0, 3.0, 4.0], "var": [0.01, 0.01, 0.01, 0.01]},
    # "1": {"mean": [1.0, 2.0, 3.0, 4.0], "var": [10.0, 10.0, 10.0, 10.0]}
    #  } }
    # note: data type of input will be changed to UINT8 automatically whatever which datatype
    # you set by parameter input_dtypes once you insert a batchnorm node befor the 1st node.
    assert (
        "means" in args.keys() and "vars" in args.keys()
    ), "means and vars must be specified together, but just got one."
    checker = (
        lambda values: len(values) > 0
        and all([len(a_set_of_value) > 0 for a_set_of_value in values])
        and all(
            [
                type(e) in [float, int]
                for a_set_of_value in values
                for e in a_set_of_value
            ]
        )
    )
    means, vars_ = args["means"], args["vars"]
    assert checker(means), "invalid means"
    assert checker(vars_), "invalid variances"
    assert len(means) == len(vars_), "the number of vars doesn't match that of means"
    mean_var_str = '{"insert_bn_before_firstnode":{'
    for i, (mean, var) in enumerate(zip(means, vars_)):
        mean_var_str += (
            '"{}":'.format(i)
            + "{"
            + '"mean":{}'.format("[" + ",".join([str(e) for e in mean]) + "]")
            + ',"var":{}'.format("[" + ",".join([str(e) for e in var]) + "]")
            + "}"
        )
        mean_var_str += "," if i < len(means) - 1 else "}"
    mean_var_str += "}"
    assert config.parse_from_string(mean_var_str).ok()
    log.info(
        "get_build_config: inserted batchnorm node before the first node successfully."
    )
    return config


def get_channel_opposite_layout(layout_in: str) -> str:
    """Get an opposite layout assoicated with the input layout.

    Args:
        layout_in(str): Input layout.

    Note:
        - Do not set input layout of network to what it already is because the function
            doesn't check the real layout of network input and cannot create a pair of
            {src: dst} with same layout ( for example nchw- > nchw). By default, we think
            you only does convert layout when it needed, and doesn't convert a layout to
            the same.

    Returns:
        - Returns an opposite layout for input layout, such as nchw for nhwc.
    """
    CORRESPONDING_LAYOUT = {"NCT": "NTC", "NCHW": "NHWC", "NCDHW": "NDHWC"}
    CORRESPONDING_LAYOUT.update({v: k for k, v in CORRESPONDING_LAYOUT.items()})
    assert layout_in in CORRESPONDING_LAYOUT.keys(), "unsupported layout"
    return CORRESPONDING_LAYOUT[layout_in]


def set_input_layout(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set layout for each input of network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    # note:
    # 1. do not set a layout with axis more than that of input to the input.
    # for example, setting layout NCHW to an input of shape [batch_size, len] with 2 axis
    # is invalid.
    # 2. do not set input layout to what it already is. for example, do not specify
    # converting input layout to nchw if the layout of input already is nchw.
    input_layouts = args["input_layout"]
    assert type(input_layouts) in [list], "invalid input layout: " + str(input_layouts)
    input_layouts = [input_layouts] if isinstance(input_layouts, str) else input_layouts
    input_layout_str = '{"convert_input_layout":{'
    for i, layout in enumerate(input_layouts):
        input_layout_str += (
            '"{}'.format(i)
            + '":{"src":"'
            + get_channel_opposite_layout(layout)
            + '",'
            + '"dst":"'
            + layout
            + '"}'
        )
        input_layout_str += "," if i < len(input_layouts) - 1 else "}"
    input_layout_str += "}"
    assert config.parse_from_string(input_layout_str).ok()
    log.info("get_build_config: set input layout successfully.")
    return config


def set_output_layout(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set layout for each output of network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    # note:
    # 1. do not set a layout with axis num more than output axis to the output.
    # for example, setting layout NCHW to an output of shape [batch_size, len] with 2 axis
    # is invalid.
    # 2. do not set output layout to what it already is.
    output_layouts = args["output_layout"]
    assert type(output_layouts) in [list], "invalid input layout"
    output_layouts = (
        [output_layouts] if isinstance(output_layouts, str) else output_layouts
    )
    output_layout_str = '{"convert_output_layout":{'
    for i, layout in enumerate(output_layouts):
        output_layout_str += (
            '"{}'.format(i)
            + '":{"src":"'
            + get_channel_opposite_layout(layout)
            + '",'
            + '"dst":"'
            + layout
            + '"}'
        )
        output_layout_str += "," if i < len(output_layouts) - 1 else "}"
    output_layout_str += "}"
    assert config.parse_from_string(output_layout_str).ok()
    log.info("get_build_config: set output layout successfully.")
    return config


def set_dynamic_or_static_input_shape(
    config: BuilderConfig, args: dict
) -> BuilderConfig:
    """Set dynamic or static input shape for the network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    if "dynamic_shape" in args.keys() and args["dynamic_shape"] is not None:
        if args["dynamic_shape"] == "true":
            assert config.parse_from_string('{"graph_shape_mutable":true}').ok()
            # cambricon-note: call set_dim_range only when dynamic_shape is true
            set_dim_range(config, args)
        elif args["dynamic_shape"] == "false":
            assert config.parse_from_string('{"graph_shape_mutable":false}').ok()
            log.info("get_build_config: fixed input shape successfully.")
        else:
            log.info("Error:the val of shape_mutable is invalid.")
    #assert config.parse_from_string('{"debug_config": {"fusion_enable": false}}').ok()
    #assert config.parse_from_string('{"debug_config": {"print_ir":  {"print_level": 1}}}').ok()
    return config


def set_dim_range(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set dim range for the network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """
    if (
        "dim_range_min" in args.keys()
        and args["dim_range_min"] is not None
        or "dim_range_max" in args.keys()
        and args["dim_range_max"] is not None
    ):
        # assert is_fixed_input_shape == False, (
        #    "already specified fixed input shape: "
        #    + str(args["input_dims"])
        #    + ", do not specify dim_range for dynamic shape again."
        # )
        assert (
            "dim_range_min" in args.keys()
            and args["dim_range_min"] is not None
            and "dim_range_max" in args.keys()
            and args["dim_range_max"] is not None
        ), "dim_range_min and dim_range_max must be provided together, but just got one of them."

        dim_range_min_list = args["dim_range_min"]
        dim_range_max_list = args["dim_range_max"]

        assert len(dim_range_min_list) == len(
            dim_range_max_list
        ), "dim_range_min does not match dim_range_max"
        dim_range_str = '{"dim_range": {'
        # an example of dim_range_str:
        # { "dim_range": {
        #       "0": {
        #           "min": [1, 224, 224, 3],
        #           "max": [12, 224, 224, 3]
        #       }
        # } }
        for i, (min_list, max_list) in enumerate(
            zip(dim_range_min_list, dim_range_max_list)
        ):
            dim_range_str += (
                '"{}":'.format(i)
                + '{"min":['
                + ",".join([str(e) for e in min_list])
                + "],"
                + '"max":['
                + ",".join([str(e) for e in max_list])
                + "]}"
            )
            dim_range_str += "," if i < len(dim_range_max_list) - 1 else "}"
        dim_range_str += "}"
        assert config.parse_from_string(dim_range_str).ok()
        log.info(
            "get_build_config: set input shape to be mutable by the specified dim_range successfully."
        )
    return config


def set_computation_preference(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set computation preference for the network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """

    assert (
        "computation_preference" in args.keys()
        and args["computation_preference"] is not None
    ), "computation_preference is not specified."

    computation_preference = args["computation_preference"]
    assert computation_preference in [
        "auto",
        "fast",
        "high_precision",
    ], "invalid computation_preference: " + str(computation_preference)
    computation_preference_str = (
        '{"computation_preference":' + '"' + computation_preference + '"' + "}"
    )
    assert config.parse_from_string(computation_preference_str).ok()
    log.info(
        "get_build_config: set computation_preference {} successfully.".format(
            computation_preference
        )
    )
    return config


def set_type64to32_conversion(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set type64to32_conversion for the network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """

    assert (
        "type64to32_conversion" in args.keys()
        and args["type64to32_conversion"] is not None
    ), "opt.type64to32_conversion is not specified."

    type64to32_conversion = args["type64to32_conversion"]
    assert type64to32_conversion in [
        "true",
        "false",
    ], "invalid type64to32_conversion: " + str(type64to32_conversion)

    # example: {"opt_config":{"type64to32_conversion":true}}
    type64to32_conversion_str = (
        '{"opt_config":{"type64to32_conversion":' + type64to32_conversion + "}}"
    )

    assert config.parse_from_string(type64to32_conversion_str).ok()
    log.info(
        "get_build_config: set opt.type64to32_conversion {} successfully.".format(
            type64to32_conversion
        )
    )
    return config


def set_conv_scale_fold(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set conv_scale_fold for the network.

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """

    assert (
        "conv_scale_fold" in args.keys() and args["conv_scale_fold"] is not None
    ), "opt.conv_scale_fold is not specified."

    conv_scale_fold = args["conv_scale_fold"]
    assert conv_scale_fold in [
        "true",
        "false",
    ], "invalid conv_scale_fold: " + str(conv_scale_fold)

    # example: {"opt_config":{"conv_scale_fold":true}}
    conv_scale_fold_str = '{"opt_config":{"conv_scale_fold":' + conv_scale_fold + "}}"

    assert config.parse_from_string(conv_scale_fold_str).ok()
    log.info(
        "get_build_config: set opt.conv_scale_fold {} successfully.".format(
            conv_scale_fold
        )
    )
    return config


def set_opt_config(config: BuilderConfig, args: dict) -> BuilderConfig:
    """Set opt_config for the network.
       In detail, set opt.type64to32_conversion, opt.conv_scale_fold

    Args:
        config(BuilderConfig): Configuration for builder.
        args(dict): All arguments.

    Returns:
        - Returns a configured BuilderConfig instance.
    """

    set_type64to32_conversion(config, args)
    set_conv_scale_fold(config, args)
    return config


def get_builder_config(args: dict) -> BuilderConfig:
    """Create a BuilderConfig for subsequent model building.

    Args:
        args(dict): All arguments.

    Returns:
        - Returns a BuilderConfig for subsequent model building.
    """
    print_parameters(
        args, MODEL_BUILDER_PARAMS, "parameters about model builder configuration:"
    )
    config = BuilderConfig()
    # set mlu architecture
    mlu_arch_list = get_mlu_arches_from_param(args)
    config.mlu_archs = mlu_arch_list
    log.info("get_build_config: set mlu architecture successfully.")

    # set precision
    assert config.parse_from_string(get_precision_json_str_from_param(args)).ok()
    log.info("get_build_config: set precision successfully.")

    if "int" in args["precision"]:
        config = set_quant_granul_and_algori(config, args)

    # set cluster num
    # the validity of cluster is not checked, please make sure the validity of which before use.
    if "cluster_num" in args.keys():
        config = set_cluster_num(config, mlu_arch_list, args)

    exist_and_not_none_checker = lambda x: x in args.keys() and args[x] is not None

    # set means and vars
    if exist_and_not_none_checker("means") or exist_and_not_none_checker("vars"):
        config = insert_bn_before_first_node(config, args)

    # set cross compiler toolchain
    if exist_and_not_none_checker("toolchain_path"):
        toolchain_path = args["toolchain_path"]
        assert toolchain_path is not None and os.path.exists(
            toolchain_path
        ), "invalid toolchain path"
        assert config.parse_from_string(
            '{"cross_compile_toolchain_path":"' + toolchain_path + '"}'
        ).ok()
        log.info("get_build_config: set cross compile toolchain path successfully.")

    # set layout for input
    if exist_and_not_none_checker("input_layout"):
        config = set_input_layout(config, args)

    # set layout for output
    if exist_and_not_none_checker("output_layout"):
        config = set_output_layout(config, args)

    # set dynamic or static input shape
    set_dynamic_or_static_input_shape(config, args)

    # cambricon-note: add set computation preference
    set_computation_preference(config, args)

    # cambricon-note: add set opt_config
    set_opt_config(config, args)

    # configure the net with the given json
    # note: for a item, the set presented in the json will override the corresponding cofiguration above
    if exist_and_not_none_checker("build_config"):
        json_path = args["build_config"]
        assert isinstance(json_path, str) and os.path.exists(
            json_path
        ), "invalid configuration json path"
        assert config.parse_from_file(json_path).ok()
        log.info(
            "get_build_config: set builder_config by json file successfully. NOTE: the items "
            "configured before has been OVERRIDED with the ones presented in the configuration json."
        )
    log.info("get_build_config: created builder configuraton successfully.")
    return config


def get_nchw_of_input(tesnor: ITensor, layout: str) -> List[int]:
    """Get batch size, channel num, height, width  and channel axis of the tensor

    Args:
        tensor(ITensor): Input tensor to get n, c, h, w and channel_axis.
        layout(str): Layout of the input tensor.

    Returns:
        - Returns a list with n, c, h, w and channel_aixs of the input tensor.
    """
    tensor_dim = tesnor.get_dimension()
    assert layout.upper() in ["NCHW", "NHWC"], "unspported layout: " + layout
    batch_size = tensor_dim.GetDimValue(0)
    get_dim = lambda i: tensor_dim.GetDimValue(i)
    if layout == "NCHW":
        return [batch_size, get_dim(1), get_dim(2), get_dim(3), 1]
    else:
        return [batch_size, get_dim(3), get_dim(1), get_dim(2), 3]


def is_rand_calib_or_fake_calib(
    net: Network, config: BuilderConfig, args: dict
) -> Tuple[bool, bool]:
    """Determine random calibration (calibrate wuth random data) or fake calibration (set the range of
    direction to [-1, 1] directly). If the calibration is  neither random calibration nor fake calibration,
    the network will be calibrated by real calibration samples from dataloader.

    Args:
        net(dict): Network to be calibrated.
        config(BuilderConfig): A BuilderConfig for calibration.
        args(dict): All parameters for calibration.

    Returns:
        - For rand_calib,returns True if calibration is random calibration, otherwise, False. For fake_calib,returns True
          for fake_calib if calibration is fake calibration, otherwise, False.
    """
    rand_calib = False
    fake_calib = False
    if "random_calib_range" in args.keys() and args["random_calib_range"] is not None:
        random_calib_range = args["random_calib_range"]
        assert (
            isinstance(random_calib_range, list) and len(random_calib_range) == 2
        ), "invalid random calibration range, expects a list with [min, max] as range calibration data"
        log.info(
            "calibration: going to calibrate network with custom range [{}]".format(
                str(random_calib_range)
            )
        )
        rand_calib = True
    elif args["file_list"] is None and args["calibration_data_path"] is None:
        log.info(
            "calibration: fill fixed quantization parameters with fake calibration"
        )
        fake_calib = True
        assert config.parse_from_string(
            '{"custom_ranges": {"": {"max": [1], "min": [-1]}}}'
        ).ok()
        log.info("calibration: set fake calibration successfully.")
    else:
        assert (
            len(args["file_list"]) > 0 and args["calibration_data_path"] is not None
        ), "Calibration list file and calibration data path must be provided together."

    input_num = net.get_input_count()
    if "file_list" in args.keys() and args["file_list"] is not None:
        img_list_file_paths = args["file_list"]
        for p in img_list_file_paths:
            assert os.path.exists(p), "file is not existing: {}".format(p)
        assert (
            "calibration_data_path" in args.keys()
            and args["calibration_data_path"] is not None
        ), "parameter calibration_data_path is not provided."
        calibration_data_path = args["calibration_data_path"]
        assert os.path.exists(
            calibration_data_path
        ), "calibration data folder {} is not existing: ".format(calibration_data_path)
        if not rand_calib and not fake_calib and len(img_list_file_paths) != input_num:
            raise ValueError(
                "calibration data should be provided for each input if calibrate"
                + " quantization with real data. expects {} img_list_file for {} inputs,".format(
                    input_num, input_num
                )
                + " but just got {}.".format(len(img_list_file_paths))
            )
    return rand_calib, fake_calib


def check_validity_of_ip(ip: str) -> bool:
    """Naive check for validity of ip address

    Args:
        ip(str): Ip address.

    Returns:
        - Returns Ture if ip address is validity, otherwise False.
    """
    parts = ip.strip().split(".")
    if len(parts) != 4:
        return False
    else:
        return all([int(e) <= 255 and int(e) >= 0 for e in parts])


def build_and_serialize(net: Network, config: BuilderConfig, args: dict) -> bool:
    """Build and serialize the magicmind model.

    Args:
        net(dict): Network to be built.
        config(BuilderConfig): A BuilderConfig for model building.
        args(dict): All parameters for model building and serialization.

    Returns:
        - Returns True if build and serialize successfully, otherwise, False.
    """
    print_parameters(
        args,
        MODEL_BUILD_AND_SERIALIZE_PARAM,
        "parameters about model building and serialization configuration:",
    )
    builder = Builder()
    PARAM_DATA_TYPE_2_MM = {
        "INT8": DataType.INT8,
        "INT16": DataType.INT16,
        "INT32": DataType.INT32,
        "INT64": DataType.INT64,
        "UINT8": DataType.UINT8,
        "UINT16": DataType.UINT16,
        "UINT32": DataType.UINT32,
        "HALF": DataType.FLOAT16,
        "FLOAT": DataType.FLOAT32,
        "BOOL": DataType.BOOL,
        "QINT8": DataType.QINT8,
        "QINT16": DataType.QINT16,
    }

    if "input_dtypes" in args.keys() and args["input_dtypes"] is not None:
        data_types = args["input_dtypes"]
        for data_type in data_types:
            assert data_type in PARAM_DATA_TYPE_2_MM.keys(), (
                "unsupported data type: " + data_type
            )
        input_num = net.get_input_count()
        assert input_num == len(
            data_types
        ), "network has {} inputs, requires {} ".format(
            input_num, input_num
        ) + "data types but just specified {}.".format(
            len(data_types)
        )
        for i in range(input_num):
            tensor = net.get_input(i)
            assert tensor.set_data_type(PARAM_DATA_TYPE_2_MM[data_types[i]]).ok()
            log.info(
                "build_and_serialize: reset input {}'s data type to: ".format(i)
                + args["input_dtypes"][i]
            )
        log.info("build_and_serialize: set input data types successfully.")

    if "output_dtypes" in args.keys() and args["output_dtypes"] is not None:
        data_types = args["output_dtypes"]
        for data_type in data_types:
            assert data_type in PARAM_DATA_TYPE_2_MM.keys(), (
                "unsupported data type: " + data_type
            )
        assert output_num == len(
            data_types
        ), "network has {} outputs, requires {} ".format(
            output_num, output_num
        ) + "data types but just specified {}.".format(
            len(data_types)
        )
        for i in range(output_num):
            tensor = net.get_output(i)
            assert tensor.set_data_type(PARAM_DATA_TYPE_2_MM[data_types[i]]).ok()
            log.info(
                "build_and_serialize: reset output {}'s data type to: ".format(i)
                + args["output_dtypes"][i]
            )
        log.info("build_and_serialize: set output data types successfully.")

    assert (
        "magicmind_model" in args.keys() and args["magicmind_model"]
    ), "path to save magicmind model is not specified."
    magicmind_model = args["magicmind_model"]
    model = builder.build_model("magicmind_model_name", net, config)
    assert isinstance(model, Model)
    log.info("build_and_serialize: built model successfully.")
    assert model.serialize_to_file(magicmind_model).ok()
    log.info("build_and_serialize: serialized model successfully.")

    input_data_types = model.get_input_data_types()
    output_data_types = model.get_output_data_types()
    input_names = model.get_input_names()
    output_names = model.get_output_names()
    input_dimensions = model.get_input_dimensions()
    output_dimensions = model.get_output_dimensions()

    MM_DATA_TYPE_2_STR = {value: key for key, value in PARAM_DATA_TYPE_2_MM.items()}
    log.info("=" * 30)
    log.info("build_and_serialize: model info:")
    for i, (name, type_, shape) in enumerate(
        zip(input_names, input_data_types, input_dimensions)
    ):
        log.info("model input[{}]'s name is {}".format(i, name))
        log.info("model input[{}]'s shape is {}".format(i, str(shape.GetDims())))
        log.info(
            "model input[{}]'s data type is {}".format(i, MM_DATA_TYPE_2_STR[type_])
        )

    for i, (name, type_, shape) in enumerate(
        zip(output_names, output_data_types, output_dimensions)
    ):
        log.info("model output[{}]'s name is {}".format(i, name))
        log.info("model output[{}]'s shape is {}".format(i, str(shape.GetDims())))
        log.info(
            "model output[{}]'s data type is {}".format(i, MM_DATA_TYPE_2_STR[type_])
        )
    log.info("=" * 30)



from insightface.recognition.arcface_torch.backbones import get_model
import argparse
import torch

def main():
    args = argparse.ArgumentParser(description='save arcface model by jit.trace')
    args.add_argument('--weights', default = './../data/models/backbone.pth',
            required=True, type = str, help = 'arcface weights')
    args.add_argument('--network', default = 'r100',
            type = str, help = 'backbone network name. see https://github.com/deepinsight/insightface/blob/master/recognition/arcface_torch for supported networks')
    args.add_argument('--input_width', dest = 'input_width', default = 112,
            type = int, help = 'model input width')
    args.add_argument('--input_height', dest = 'input_height', default = 112,
            type = int, help = 'model input height')
    args.add_argument('--output_pt', default = './../data/models/arcface.pt',
            type = str, help = 'output pt path')

    args = args.parse_args()

    input_data = torch.randn(1, 3, args.input_height, args.input_width).float()
    net = get_model(args.network, fp16=False)
    net.load_state_dict(torch.load(args.weights, map_location='cpu'))
    net.eval()
    traced = torch.jit.trace(net, input_data)
    torch.jit.save(traced, args.output_pt)
    print('torchscript saved to', args.output_pt)

if __name__ == "__main__":
    main()




import magicmind.python.runtime as mm
import numpy as np
from mm_runner import MMRunner
# used to init decoder inputs
def load_decoder_input_model(device_id):
    dtype = mm.DataType.FLOAT32

    network = mm.Network()
    zero_const = network.add_i_const_node(dtype, mm.Dims([]), [0])
    zero_const_int = network.add_i_const_node(mm.DataType.INT32, mm.Dims([]), [0])
    one_const_1_dim = network.add_i_const_node(mm.DataType.INT32, mm.Dims([1]), [1])
    zero_const_1_dim = network.add_i_const_node(mm.DataType.INT32, mm.Dims([1]), [0])
    memory_size = network.add_input(mm.DataType.INT32, mm.Dims([1]))
    sequence_lengths = network.add_input(mm.DataType.INT32)
    max_sequence_len = network.add_input(mm.DataType.INT32, mm.Dims([1]))

    '''
    mask, implementation:

    max_len = torch.max(lengths).item()
    ids = torch.arange(0, max_len, device=lengths.device, dtype=lengths.dtype)
    mask = (ids < lengths.unsqueeze(1)).byte()
    mask = torch.le(mask, 0)
    '''
    ids = network.add_i_range_node(zero_const_1_dim.get_output(0), max_sequence_len, one_const_1_dim.get_output(0))
    sequence_lengths = network.add_i_unsqueeze_node(sequence_lengths, one_const_1_dim.get_output(0))
    mask = network.add_i_logic_node(ids.get_output(0), sequence_lengths.get_output(0), mm.enums.ILogic.LT)
    mask = network.add_i_cast_node(mask.get_output(0), mm.DataType.INT32)
    mask = network.add_i_logic_node(mask.get_output(0), zero_const_int.get_output(0), mm.enums.ILogic.LE)

    # fill tensors with zero
    memory = network.add_i_fill_node(memory_size, zero_const.get_output(0))

    network.mark_output(memory.get_output(0))
    network.mark_output(mask.get_output(0))

    builder = mm.Builder()
    builder_config = mm.BuilderConfig()
    model = builder.build_model("decoder_input", network, builder_config)
    assert model is not None, 'build decoder input model failed'
    assert model.serialize_to_file("./decoder_input_model").ok()
    return MMRunner(mm_file="./decoder_input_model", device_id=device_id)

# mel_outputs = mel_outputs[:,:,:torch.max(mel_lengths)] mm implementation
# input1: mel_outputs from decoder
# input2: mel_lengths from decoder
# output: mels(postnet input)
def load_melcrop_model(device_id):
    network = mm.Network()
    mel_inputs = network.add_input(mm.DataType.FLOAT32)
    mel_lengths = network.add_input(mm.DataType.INT32)
    # torch.max(mel_lengths) # mm reduce max with axis=0
    max_axis = network.add_i_const_node(dtype=mm.DataType.INT32, dimensions=mm.Dims([1]), value=np.array([0], dtype=np.int32)).get_output(0)
    reduce_max = network.add_i_reduce_node(mel_lengths, max_axis, mm.IReduce.MAX, False).get_output(0)
    # mel_outputs[:,:,:max_len], mm strided slice with axis=2, stride=1
    begin_idx = max_axis  # 0
    slice_axis = network.add_i_const_node(dtype=mm.DataType.INT32, dimensions=mm.Dims([1]), value=np.array([2], dtype=np.int32)).get_output(0)
    mel_outputs = network.add_i_slice_node(mel_inputs, begin_idx, reduce_max, slice_axis).get_output(0)
    assert network.mark_output(mel_outputs).ok()

    # build
    config = mm.BuilderConfig()
    assert config.parse_from_string('{"precision_config":{"precision_mode":"force_float32"}}').ok()
    assert config.parse_from_string('{"archs":["mtp_372"]}').ok()
    builder = mm.Builder()
    assert builder is not None
    model = builder.build_model('melcrop', network, config)
    assert model is not None, 'build melcrop model failed'
    assert model.serialize_to_file("./melcrop_model").ok()
    return MMRunner(mm_file="./melcrop_model", device_id=device_id)



import json
import os
import numpy as np
import cv2
import argparse
import prettytable
ROOT_DIR = os.path.abspath(os.path.join(os.path.split(__file__)[0], "../"))

parser = argparse.ArgumentParser()
parser.add_argument("--model", type=str)
parser.add_argument("--output_file", type=str)
parser.add_argument("--output_ok_file", type=str)
parser.add_argument("--metric", type=str, choices = ["top1", "top1andtop5", "cocomAP", "vocmAP", "unet", "squad", "voc_miou", "1e5and1e4", "mrpc", "u2net", "cocoKeyPoints", "total_text"])

ROOT_DIR = os.path.abspath(os.path.join(os.path.split(__file__)[0], ""))
report_file = "%s/eval_report.csv"%ROOT_DIR
# top1 compare
def compare_top1(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    old_top1 = []
    new_top1 = []
    for l1 in lines1:
        new_top1.append(float(l1.split(":")[1].strip()))
    for l2 in lines2:
        old_top1.append(float(l2.split(":")[1].strip()))
    status = new_top1[0] >= old_top1[0]
    top1_diff = new_top1[0] - old_top1[0]
    return status, top1_diff

# top1andtop5 compare
def compare_top1andtop5(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    old_top1andtop5 = []
    new_top1andtop5 = []
    for l1 in lines1:
        new_top1andtop5.append(float(l1.split(":")[1].strip()))
    for l2 in lines2:
        old_top1andtop5.append(float(l2.split(":")[1].strip()))
    status = new_top1andtop5[0] >= old_top1andtop5[0] and new_top1andtop5[1] >= old_top1andtop5[1]
    top1_diff = new_top1andtop5[0] - old_top1andtop5[0]
    top5_diff = new_top1andtop5[1] - old_top1andtop5[1]
    return status, top1_diff, top5_diff

# coco mAP
def compute_cocomAP(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    for line1 in lines1:
        if "Average Precision  (AP) @[ IoU=0.50:0.95 | area=   all | maxDets=100 ]" in line1:
            new_mAP_0_5to0_95 = float(line1.split("=")[-1].strip())
        if "Average Precision  (AP) @[ IoU=0.50      | area=   all | maxDets=100 ]" in line1:
            new_mAP_0_5 = float(line1.split("=")[-1].strip())
    for line2 in lines2:
        if "Average Precision  (AP) @[ IoU=0.50:0.95 | area=   all | maxDets=100 ]" in line2:
            old_mAP_0_5to0_95 = float(line2.split("=")[-1].strip())
        if "Average Precision  (AP) @[ IoU=0.50      | area=   all | maxDets=100 ]" in line2:
            old_mAP_0_5 = float(line2.split("=")[-1].strip())
    status = new_mAP_0_5to0_95 >= old_mAP_0_5to0_95 and new_mAP_0_5 >= old_mAP_0_5
    mAP_0_5to0_95_diff = new_mAP_0_5to0_95 - old_mAP_0_5to0_95
    mAP_0_5_diff = new_mAP_0_5 - old_mAP_0_5
    return status, mAP_0_5to0_95_diff, mAP_0_5_diff

def compute_vocmAP(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    for line1 in lines1:
        if "Mean AP" in line1:
            new_mAP = float(line1.split("=")[-1].strip())
    for line2 in lines2:
        if "Mean AP" in line2:
            old_mAP = float(line2.split("=")[-1].strip())
    status = new_mAP >= old_mAP
    mAP_diff = new_mAP - old_mAP
    return status, mAP_diff

def compute_unet(file1, file2):
    f1_dict=json.load(open(file1, "r"))
    f2_dict=json.load(open(file2, "r"))
    new_accuracy = f1_dict["results"]["mean"]["1"]["Accuracy"]
    new_dice = f1_dict["results"]["mean"]["1"]["Dice"]
    new_precision = f1_dict["results"]["mean"]["1"]["Precision"]
    new_recall = f1_dict["results"]["mean"]["1"]["Recall"]
    old_accuracy = f2_dict["results"]["mean"]["1"]["Accuracy"]
    old_dice = f2_dict["results"]["mean"]["1"]["Dice"] 
    old_precision = f2_dict["results"]["mean"]["1"]["Precision"]
    old_recall = f2_dict["results"]["mean"]["1"]["Recall"]
    status = new_accuracy >= old_accuracy
    accuracy_diff = new_accuracy - old_accuracy
    dice_diff = new_dice - old_dice
    precision_diff = new_precision - old_precision
    recall_diff = new_recall - old_recall
    return status, accuracy_diff, dice_diff, precision_diff, recall_diff

def compute_squad(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    for l1 in lines1:
        new_exact = float(l1.split(":")[-1].split(",")[1].strip(")"))
        new_f1 = float(l1.split(":")[-1].split(",")[3].strip(")"))
    for l2 in lines2:
        old_exact = float(l2.split(":")[-1].split(",")[1].strip(")"))
        old_f1 = float(l2.split(":")[-1].split(",")[3].strip(")"))
    status = new_exact >= old_exact and new_f1 >= old_f1
    exact_diff = new_exact - old_exact
    f1_diff = new_f1 - old_f1
    return status, exact_diff, f1_diff

def compute_mrpc(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
        
    new_acc = float(lines1[0].split(":")[-1])
    new_f1 =  float(lines1[1].split(":")[-1])
    old_acc = float(lines2[0].split(":")[-1])
    old_f1 =  float(lines2[1].split(":")[-1])
    
    status = new_acc >= old_acc and new_f1 >= old_f1
    acc_diff = new_acc - old_acc
    f1_diff = new_f1 - old_f1
    return status, acc_diff, f1_diff
    
def compute_voc_miou(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    for l1 in lines1:
        if "miou" in l1:
            new_voc_miou = float(l1.split(",")[4])
    for l2 in lines2:
        if "miou" in l2:
            old_voc_miou = float(l2.split(",")[4])
    voc_miou_diff = new_voc_miou - old_voc_miou
    status = new_voc_miou >= old_voc_miou
    return status, voc_miou_diff

def compute_total_text(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    new_precision = float(lines1[0].split(" ")[2])
    new_recall = float(lines1[1].split(" ")[2])
    new_fmeasure = float(lines1[2].split(" ")[2])
    old_precision = float(lines2[0].split(" ")[2])
    old_recall =  float(lines2[1].split(" ")[2])
    old_fmeasure = float(lines2[2].split(" ")[2])


    status = new_precision >= old_precision
    precision_diff = new_precision - old_precision
    return status, precision_diff

def summary_top1(testcase_name, file, metric, result, top1_diff):
    header = ["testcase", "file", "metric", "status", "top1_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_top1_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-1], metric, status, top1_diff])
    tb.add_row ([testcase_name ,file.split("/")[-1], metric, status, top1_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_top1andtop5(testcase_name, file, metric, result, top1_diff, top5_diff):
    header = ["testcase", "file", "metric", "status", "top1_diff", "top5_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_top1andtop5_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"
    
    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, top1_diff, top5_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, top1_diff, top5_diff])
    
    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())
def summary_1e5and1e4(testcase_name, file, metric, result, res_1e5_diff, res_1e4_diff):
    header = ["testcase", "file", "metric", "status", "1e5_diff", "1e4_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_1e5and1e4_report.csv"%ROOT_DIR

    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"
    
    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, res_1e5_diff, res_1e4_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, res_1e5_diff, res_1e4_diff])
   
    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_u2net(testcase_name, file, metric, result, mae, fmeasure):
    header = ["testcase", "file", "metric", "status", "average_mae_diff", "max_fmeasure_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_u2net_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"
    
    show_tb.add_row ([testcase_name, file.split("/")[-1], metric, status, mae, fmeasure])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, mae, fmeasure])
 
    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_cocoKeyPoints(testcase_name, file, metric, result, body25_diff, coco_diff):
    header = ["testcase", "file", "metric", "status", "body25_diff", "coco_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_cocoKeyPoints_report.csv"%ROOT_DIR

    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"
    
    show_tb.add_row ([testcase_name, file.split("/")[-1], metric, status, body25_diff, coco_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, body25_diff, coco_diff])
    
    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_cocomAP(testcase_name, file, metric, result, mAP_0_5to0_95_diff, mAP_0_5_diff):
    header = ["testcase", "file", "metric", "status", "mAP_0_5to0_95_diff", "mAP_0_5_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_cocomAP_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, mAP_0_5to0_95_diff, mAP_0_5_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, mAP_0_5to0_95_diff, mAP_0_5_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_vocmAP(testcase_name, file, metric, result, mAP_diff):
    header = ["testcase", "file", "metric", "status", "mAP_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_vocmAP_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-3], metric, status, mAP_diff])
    tb.add_row ([testcase_name ,file.split("/")[-3], metric, status, mAP_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_unet(testcase_name, file, metric, result, accuracy_diff, dice_diff, precision_diff, recall_diff):
    header = ["testcase", "file", "metric", "status", "accuracy_diff", "dice_diff", "precision_diff", "recall_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_unet_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, accuracy_diff, dice_diff, precision_diff, recall_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, accuracy_diff, dice_diff, precision_diff, recall_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_squad(testcase_name, file, metric, result, exact_diff, f1_diff):
    header = ["testcase", "file", "metric", "status", "exact_diff", "f1_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_squad_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, exact_diff, f1_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, exact_diff, f1_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_mrpc(testcase_name, file, metric, result, acc_diff, f1_diff):
    header = ["testcase", "file", "metric", "status", "acc_diff", "f1_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_squad_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, acc_diff, f1_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, acc_diff, f1_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_voc_miou(testcase_name, file, metric, result, voc_miou_diff):
    header = ["testcase", "file", "metric", "status", "voc_miou_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_voc_miou_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-1], metric, status, voc_miou_diff])
    tb.add_row ([testcase_name ,file.split("/")[-1], metric, status, voc_miou_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary_total_text(testcase_name, file, metric, result, precision_diff):
    header = ["testcase", "file", "metric", "status", "precision_diff"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    report_file = "%s/eval_total_text_report.csv"%ROOT_DIR
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if result:
        status = "pass"
    else:
        status = "fail"

    show_tb.add_row ([testcase_name, file.split("/")[-2], metric, status, precision_diff])
    tb.add_row ([testcase_name ,file.split("/")[-2], metric, status, precision_diff])

    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

def summary(testcase_name, file, metric, result):
    header = [ "testcase" , "file" , "metric", "similar", "status"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    show_tb.add_row ([ testcase_name , file, metric, result, "fail"])
    tb.add_row ([ testcase_name , file, metric, result, "fail"])

    print(show_tb)
    with open(report_file, 'w') as f:
        f.write(tb.get_csv_string())

if __name__ == "__main__":
    args = parser.parse_args()
    testcase_name = args.model    
    output_file = os.path.join(args.output_file)
    output_ok_file = os.path.join(args.output_ok_file)
    if not os.path.exists(output_file):
        result = "file: %s not exists."%output_file
        summary(testcase_name, args.output_file, args.metric, result)
    elif not os.path.exists(output_ok_file):
        result = "file: %s not exists."%output_ok_file
        summary(testcase_name, args.output_file, args.metric, result)
    else:
        if args.metric == "top1":
            result, top1_diff = compare_top1(output_file, output_ok_file)
            summary_top1(testcase_name, args.output_file, args.metric, result, top1_diff)
        elif args.metric == "top1andtop5":
            result, top1_diff, top5_diff = compare_top1andtop5(output_file, output_ok_file)
            summary_top1andtop5(testcase_name, args.output_file, args.metric, result, top1_diff, top5_diff)
        elif args.metric == "1e5and1e4":
            result, top1_diff, top5_diff = compare_top1andtop5(output_file, output_ok_file)
            summary_1e5and1e4(testcase_name, args.output_file, args.metric, result, top1_diff, top5_diff)
        elif args.metric == "cocomAP":
            result, mAP_0_5to0_95_diff, mAP_0_5_diff = compute_cocomAP(output_file, output_ok_file)
            summary_cocomAP(testcase_name, args.output_file, args.metric, result, mAP_0_5to0_95_diff, mAP_0_5_diff)
        elif args.metric == "cocoKeyPoints":
            result, body25_diff, coco_diff = compare_top1andtop5(output_file, output_ok_file)
            summary_cocoKeyPoints(testcase_name, args.output_file, args.metric, result, body25_diff, coco_diff)
        elif args.metric == "vocmAP":
            result, mAP_diff = compute_vocmAP(output_file, output_ok_file)
            summary_vocmAP(testcase_name, args.output_file, args.metric, result, mAP_diff)
        elif args.metric == "unet":
            result, accuracy_diff, dice_diff, precision_diff, recall_diff = compute_unet(output_file, output_ok_file)
            summary_unet(testcase_name, args.output_file, args.metric, result, accuracy_diff, dice_diff, precision_diff, recall_diff)
        elif args.metric == "u2net":
            result, mae, fmeasure = compare_top1andtop5(output_file, output_ok_file)
            summary_u2net(testcase_name, args.output_ok_file, args.metric, result, mae, fmeasure)
        elif args.metric == "squad":
            result, exact_diff, f1_diff = compute_squad(output_file, output_ok_file)
            summary_squad(testcase_name, args.output_file, args.metric, result, exact_diff, f1_diff)
        elif args.metric == "voc_miou":
            result, voc_miou_diff = compute_voc_miou(output_file, output_ok_file)
            summary_voc_miou(testcase_name, args.output_ok_file, args.metric, result, voc_miou_diff)
        elif args.metric == "mrpc":
            result, acc_diff, f1_diff = compute_mrpc(output_file, output_ok_file)
            summary_mrpc(testcase_name, args.output_file, args.metric, result, acc_diff, f1_diff)
        elif args.metric == "total_text":
            result, precision_diff = compute_total_text(output_file, output_ok_file)
            summary_total_text(testcase_name, args.output_file, args.metric, result, precision_diff)
        else:
            result = "metric: %s not support."%args.metric
            summary(testcase_name, args.output_file, args.metric, result)




import magicmind.python.runtime as mm
import os
import time
from criteo_reader import RecDataset
from logger import Logger
from calibrator import CalibData
import paddle
from paddle.io import DataLoader

log = Logger()

from framework_parser import OnnxParser
from build_param import get_argparser
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)

def get_network(args):
    parser = OnnxParser(args)
    network = parser.parse()
    return network

def get_args():
    arg_parser = get_argparser()
    return arg_parser.parse_args()

def calibrate(args, network: mm.Network, config: mm.BuilderConfig): 
    sparse_inputs_slots = 27
    dense_input_dim=13
    # 创建量化工具并设置量化统计算法
    sample_data = []
    data_dir = args.calibration_data_path
    batch_size = args.input_dims[0][0]
    MAX_SAMPLES = batch_size
    file_list = [os.path.join(data_dir, x) for x in os.listdir(data_dir)]
    dataset = RecDataset(file_list)
    place = paddle.set_device('cpu')
    test_dataloader= DataLoader(
                        dataset,
                        batch_size=batch_size,
                        places=place,
                        drop_last=True,
                        num_workers=0)
    input_data = []
    for batch_id, batch in enumerate(test_dataloader()):
        batch_numpy_list = [tensor.numpy() for tensor in batch]
        input_data = batch_numpy_list[1:]
        if batch_id == 0:
            break
    for i in range(sparse_inputs_slots-1):
        sample_data.append(CalibData(mm.Dims((batch_size, 1)), i, MAX_SAMPLES,input_data))
    sample_data.append(CalibData(mm.Dims((batch_size, dense_input_dim)), sparse_inputs_slots-1, MAX_SAMPLES,input_data))
    calibrator = mm.Calibrator(sample_data)
    #assert calibrator is not None
    # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQM_ALGORITHM）。
    assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
    assert calibrator.calibrate(network, config).ok()

def main():
    begin_time = time.time()
    args = get_args()
    network = get_network(args)

    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)

    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)
    assert build_config.parse_from_string('{"debug_config": {"fusion_enable": false}}').ok()

    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)
    log.info("Build model...")
    
    # 生成模型并导出
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))


if __name__ == "__main__":
    main()









 



from pycocotools.coco import COCO
from pycocotools.cocoeval import COCOeval
import glob
import os
import argparse
import json
import time
import sys
sys.path.append(os.path.join(os.getenv("MAGICMIND_CLOUD"), "test"))
from record_result import write_result
class_dict = { 'person':1,'bicycle':2,'car':3,
               'motorbike':4,'aeroplane':5,'bus':6,
               'train':7,'truck':8,'boat':9,
               'traffic light':10,'fire hydrant':11,'stop sign':13,
               'parking meter':14,'bench':15,'bird':16,
               'cat':17,'dog':18,'horse':19,
               'sheep':20,'cow':21,'elephant':22,
               'bear':23,'zebra':24,'giraffe':25,
               'backpack':27,'umbrella':28,'handbag':31,
               'tie':32,'suitcase':33,'frisbee':34,
               'skis':35,'snowboard':36,'sports ball':37,
               'kite':38,'baseball bat':39,'baseball glove':40,
               'skateboard':41,'surfboard':42,'tennis racket':43,
               'bottle':44,'wine glass':46,'cup':47,
               'fork':48,'knife':49,'spoon':50,
               'bowl':51,'banana':52,'apple':53,
               'sandwich':54,'orange':55,'broccoli':56,
               'carrot':57,'hot dog':58,'pizza':59,
               'donut':60,'cake':61,'chair':62,
               'sofa':63,'pottedplant':64,'bed':65,
               'diningtable':67,'toilet':70,'tvmonitor':72,
               'laptop':73,'mouse':74,'remote':75,
               'keyboard':76,'cell phone':77,'microwave':78,
               'oven':79,'toaster':80,'sink':81,
               'refrigerator':82,'book':84,'clock':85,
               'vase':86,'scissors':87,'teddy bear':88,
               'hair drier':89,'toothbrush':90 }

def get_args():
    """
    This function creates a command-line argument parser and returns parsed arguments

    :returns: parsed command-line arguments (in the form of argparse.Namespace)
    """
    parser = argparse.ArgumentParser(description='Calculate the mAP of coco dataset')

    parser.add_argument("--file_list", dest = 'file_list', help =
                        "File list that use to calculate mAP",
                        default = "./file_list", type = str)
    parser.add_argument("--result_dir", dest = "result_dir", help =
                        "The inference result of the file, gernerate json file by results",
                        default = './',type = str)
    parser.add_argument("--ann_dir", dest = "ann_dir", help =
                        "The annotation file directory",
                        default = './',type = str)
    parser.add_argument("--data_type", dest = "data_type", help =
                        "The data type. e.g. val2014, val2015, val2017",
                        default = 'val2017',type = str)
    parser.add_argument("--json_name", dest = "json_name", help =
                        "name of the output file(.json)",
                        default = 'results',type = str)
    parser.add_argument("--img_dir", dest = "img_dir", help = "coco datasets path", default = "../datasets/", type = str)
    parser.add_argument("--image_num", dest = "image_num", help = "image number", default = 10, type = int)
    parser.add_argument("--infer_mode", dest = "infer_mode", help = "infer_mode which used to infer model", default = "infer_python", type = str)
    return parser.parse_args()

def parse_output(input):
    objs = []
    f = open(input)
    for line in f.readlines():
        objs.append(line.split(','))
    f.close()
    return objs

def get_bbox(result_objs):

    bbox_data = [float(value) for value in result_objs[2:]]
    #bbox_res = [bbox_data[0] * bbox_data[4],
    #            bbox_data[1] * bbox_data[5],
    #            (bbox_data[2] - bbox_data[0]) * bbox_data[4],
    #            (bbox_data[3] - bbox_data[1]) * bbox_data[5]]
    bbox_res = [bbox_data[0] ,
                bbox_data[1] ,
                (bbox_data[2] - bbox_data[0]) ,
                (bbox_data[3] - bbox_data[1])]
    return bbox_res

def generate_json_file(file_list, result_dir, clases_list, output_file, img_dir, image_num = -1):

    img_list_full = []
    f = open(file_list)
    num = 0 
    for img in f.readlines():
        img = os.path.join(img_dir, img)
        img_list_full.append(img)
        num += 1
        if num >= image_num and image_num != -1:
            break
    f.close()

    json_file_name = '%s.json'%(output_file)
    img_ids = []

    with open(json_file_name,'w+') as fp:
        fp.write("[\n")

        for ii, img in enumerate(img_list_full):
            img_name = os.path.splitext(os.path.basename(img))[0]
            index = int(img_name.split('_')[-1].lstrip('0'))
            img_ids.append(index)
            result_objs = parse_output(result_dir + '/' + img_name + '.txt')

            for i in range(len(result_objs)):
                img_id = index
                category_id = class_dict[result_objs[i][0]]
                score = float(result_objs[i][1])
                bbox = get_bbox(result_objs[i])
                json.dump({'image_id':img_id , 'category_id':category_id, 'bbox':bbox, 'score':score}, fp)
                if ii == len(img_list_full) - 1 and i == len(result_objs) - 1:
                    fp.write("]\n")
                else:
                    fp.write(",\n")

    return img_ids, json_file_name

def update_json_meanAp(json_data, meanAp, key):
    if isinstance(json_data, dict):
        for k in json_data:
            if k == key:
               json_data[k] = meanAp
            elif isinstance(json_data[k], dict):
               update_json_meanAp(json_data[k], meanAp, key)

class redirect:
    content = ""
    def write(self, str):
        self.content += str
    def flush(self):
        self.content = ""

if __name__ == "__main__":
    args = get_args()
    img_list = args.file_list
    result_dir = args.result_dir
    json_file  = args.json_name
    img_dir = args.img_dir

    result_dir = result_dir.rstrip('/')

    img_ids, res_file = generate_json_file(img_list, result_dir, class_dict, json_file, img_dir, args.image_num)

    print('num of images: %d'%(len(img_ids)))
    time.sleep(2)

    ann_type = ['segm', 'bbox', 'keypoints']
    ann_type = ann_type[1]
    prefix = 'person_keypoints' if ann_type == 'keypoints' else 'instances'

    data_dir = args.ann_dir
    data_type = args.data_type
    ann_file = '%s/annotations/%s_%s.json'%(data_dir, prefix, data_type)
    coco_gt = COCO(ann_file)

    #res_file = 'instances_val2014_fakebbox100_results.json'
    coco_dt = coco_gt.loadRes(res_file)

    coco_eval = COCOeval(coco_gt, coco_dt, ann_type)
    coco_eval.params.imgIds = img_ids
    coco_eval.evaluate()
    coco_eval.accumulate()
    coco_eval.summarize()
    write_result(**{"infer_mode": args.infer_mode, "dataset": "coco", "metric":"map", "eval": coco_eval.stats[1]})
    input_json_file = os.getenv('OUTPUT_JSON_FILE','')
    if os.path.isfile(input_json_file):
        r = redirect()
        sys.stdout = r
        coco_eval.summarize()
        file_in = open(input_json_file, "r")
        json_data = json.load(file_in)
        update_json_meanAp(json_data, r.content, 'meanAp')
        file_in.close()
        file_out = open(input_json_file, "w")
        json.dump(json_data, file_out,indent=2)
        file_out.close()
        r.flush()



# Copyright (c) 2020 PaddlePaddle Authors. All Rights Reserved. 
#   
# Licensed under the Apache License, Version 2.0 (the "License");   
# you may not use this file except in compliance with the License.  
# You may obtain a copy of the License at   
#   
#     http://www.apache.org/licenses/LICENSE-2.0    
#   
# Unless required by applicable law or agreed to in writing, software   
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
# See the License for the specific language governing permissions and   
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

from logger import setup_logger
logger = setup_logger(__name__)

__all__ = ['get_categories']


def get_categories(metric_type, anno_file=None, arch=None):
    """
    Get class id to category id map and category id
    to category name map from annotation file.

    Args:
        metric_type (str): metric type, currently support 'coco', 'voc', 'oid'
            and 'widerface'.
        anno_file (str): annotation file path
    """
    if arch == 'keypoint_arch':
        return (None, {'id': 'keypoint'})

    if anno_file == None or (not os.path.isfile(anno_file)):
        logger.warning(
            "anno_file '{}' is None or not set or not exist, "
            "please recheck TrainDataset/EvalDataset/TestDataset.anno_path, "
            "otherwise the default categories will be used by metric_type.".
            format(anno_file))

    if metric_type.lower() == 'coco' or metric_type.lower(
    ) == 'rbox' or metric_type.lower() == 'snipercoco':
        if anno_file and os.path.isfile(anno_file):
            if anno_file.endswith('json'):
                # lazy import pycocotools here
                from pycocotools.coco import COCO
                coco = COCO(anno_file)
                cats = coco.loadCats(coco.getCatIds())

                clsid2catid = {i: cat['id'] for i, cat in enumerate(cats)}
                catid2name = {cat['id']: cat['name'] for cat in cats}

            elif anno_file.endswith('txt'):
                cats = []
                with open(anno_file) as f:
                    for line in f.readlines():
                        cats.append(line.strip())
                if cats[0] == 'background': cats = cats[1:]

                clsid2catid = {i: i for i in range(len(cats))}
                catid2name = {i: name for i, name in enumerate(cats)}

            else:
                raise ValueError("anno_file {} should be json or txt.".format(
                    anno_file))
            return clsid2catid, catid2name

        # anno file not exist, load default categories of COCO17
        else:
            if metric_type.lower() == 'rbox':
                logger.warning(
                    "metric_type: {}, load default categories of DOTA.".format(
                        metric_type))
                return _dota_category()
            logger.warning("metric_type: {}, load default categories of COCO.".
                           format(metric_type))
            return _coco17_category()

    elif metric_type.lower() == 'voc':
        if anno_file and os.path.isfile(anno_file):
            cats = []
            with open(anno_file) as f:
                for line in f.readlines():
                    cats.append(line.strip())

            if cats[0] == 'background':
                cats = cats[1:]

            clsid2catid = {i: i for i in range(len(cats))}
            catid2name = {i: name for i, name in enumerate(cats)}

            return clsid2catid, catid2name

        # anno file not exist, load default categories of
        # VOC all 20 categories
        else:
            logger.warning("metric_type: {}, load default categories of VOC.".
                           format(metric_type))
            return _vocall_category()

    elif metric_type.lower() == 'oid':
        if anno_file and os.path.isfile(anno_file):
            logger.warning("only default categories support for OID19")
        return _oid19_category()

    elif metric_type.lower() == 'widerface':
        return _widerface_category()

    elif metric_type.lower() == 'keypointtopdowncocoeval' or metric_type.lower(
    ) == 'keypointtopdownmpiieval':
        return (None, {'id': 'keypoint'})

    elif metric_type.lower() in ['mot', 'motdet', 'reid']:
        if anno_file and os.path.isfile(anno_file):
            cats = []
            with open(anno_file) as f:
                for line in f.readlines():
                    cats.append(line.strip())
            if cats[0] == 'background':
                cats = cats[1:]
            clsid2catid = {i: i for i in range(len(cats))}
            catid2name = {i: name for i, name in enumerate(cats)}
            return clsid2catid, catid2name
        # anno file not exist, load default category 'pedestrian'.
        else:
            logger.warning(
                "metric_type: {}, load default categories of pedestrian MOT.".
                format(metric_type))
            return _mot_category(category='pedestrian')

    elif metric_type.lower() in ['kitti', 'bdd100kmot']:
        return _mot_category(category='vehicle')

    elif metric_type.lower() in ['mcmot']:
        if anno_file and os.path.isfile(anno_file):
            cats = []
            with open(anno_file) as f:
                for line in f.readlines():
                    cats.append(line.strip())
            if cats[0] == 'background':
                cats = cats[1:]
            clsid2catid = {i: i for i in range(len(cats))}
            catid2name = {i: name for i, name in enumerate(cats)}
            return clsid2catid, catid2name
        # anno file not exist, load default categories of visdrone all 10 categories
        else:
            logger.warning(
                "metric_type: {}, load default categories of VisDrone.".format(
                    metric_type))
            return _visdrone_category()

    else:
        raise ValueError("unknown metric type {}".format(metric_type))



# -*- coding: utf-8 -*-
from magicmind.python.runtime.parser import Parser
import magicmind.python.runtime as mm
import cv2
import numpy as np
import os
import glob
import argparse

def do_calibrate(network, calib_data, config, precision):
    calibrator = mm.Calibrator([calib_data])
    assert calibrator is not None

    assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
    assert config.parse_from_string(
        """{"precision_config": {"precision_mode": "%s"}}"""%(precision)).ok()
    # calibrate the network
    assert calibrator.calibrate(network, config).ok()

def preprocess_image(img, dst_shape) -> np.ndarray:

    img = cv2.resize(img, (dst_shape[0], dst_shape[1]))
    img = img.astype(dtype = np.float32)
    return img

class FixedCalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(1), self.shape_.GetDimValue(2))

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            img = preprocess_image(img, self.dst_shape_)
            imgs.append(img[np.newaxis,:])
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()

def construct_model(args, reload_by_serilize=True):
    # init builder, network, builder_config and parser
    builder = mm.Builder()
    network = mm.Network()
    config = mm.BuilderConfig()
    parser = Parser(mm.ModelKind.kTensorflow)

    # get input dims from network
    parser.set_model_param("tf-model-type", "tf-graphdef-file")
    parser.set_model_param("tf-graphdef-inputs", ["input_images"])
    parser.set_model_param("tf-graphdef-outputs", ["Sigmoid"])
    
    assert parser.parse(network, args.tf_pb).ok()
    assert network.get_input(0).set_dimension(mm.Dims((args.batch_size, args.img_size[0], args.img_size[1], 3))).ok()
    config.parse_from_string('{"archs":[{"mtp_372": [2,6,8]}]}')
    config.parse_from_string('{"opt_config":{"type64to32_conversion": true}}')
    config.parse_from_string('{"opt_config":{"conv_scale_fold": true}}')
    if args.shape_mutable=='true':
        assert config.parse_from_string('{"graph_shape_mutable": true}').ok()
        assert config.parse_from_string('{"dim_range": {"0": {"min": [1, 704, 1216, 3], "max": [32, 704, 1216,3]}}}').ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable": false}').ok()
    
    # create calibrate data
    calib_data = FixedCalibData(mm.Dims([args.batch_size, args.img_size[0], args.img_size[1], 3]),
                                max_samples = args.batch_size,
                                img_dir = args.datasets_dir)
    
    do_calibrate(network, calib_data, config, args.precision)
    model = builder.build_model("quanmodel", network, config)
    assert model.serialize_to_file(args.mm_model).ok()
    print(args.mm_model," was saved successfully in the current path.")

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--tf_pb', type=str, default='../data/models/psenet.pb', help='tf_pb path(s)')
    parser.add_argument('--mm_model', type=str, default='../data/', help='saved .mm model name')
    parser.add_argument('--precision', type=str, default='force_float32', help='precision')
    parser.add_argument('--shape_mutable', type=str, default="false", required=True ,help='shape_mutable')
    parser.add_argument('--datasets_dir', type=str, default='./icdar2015/images/', help='quantized data path,default: /data/datasets/COCO2017/images/val2017')
    parser.add_argument('--quan_img_num', type=int, default=5, help='quantized img num, default:10')
    parser.add_argument('--img_size', type=list, default=[704, 1216], help='inference size (pixels)')
    parser.add_argument('--batch_size', type=int, default=1, help='inference batch, default:1') 
    args = parser.parse_args()    

    construct_model(args)

    



import numpy as np
import magicmind.python.runtime as mm
import cv2
from typing import List
import os
import glob

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_list):
        super().__init__()
        self.data_paths_ = glob.glob(img_list + "/images/*/*.jpg")
        # print("calibrate samples : ", len(self.data_paths_))
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_

    def letterbox(self, img):
        src_h, src_w = img.shape[0], img.shape[1]
        dst_h, dst_w = self.shape_.GetDimValue(2), self.shape_.GetDimValue(3)
        ratio = min(dst_h / src_h, dst_w / src_w)
        unpad_h, unpad_w = int(round(src_h * ratio)), int(round(src_w * ratio))
        img = cv2.resize(img, (unpad_w, unpad_h), interpolation=cv2.INTER_LINEAR)
        # padding
        pad_t = int(round((dst_h - unpad_h) / 2))
        pad_b = dst_h - unpad_h - pad_t
        pad_l = int(round((dst_w - unpad_w) / 2))
        pad_r = dst_w - unpad_w - pad_l
        img = cv2.copyMakeBorder(img, pad_t, pad_b, pad_l, pad_r, cv2.BORDER_CONSTANT, value=(128, 128, 128))
        return img

    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            # resize as letterbox
            img = self.letterbox(img)
            img = np.float32(img)
            img -= (104, 117, 123)
            # HWC to CHW
            img = img.transpose(2, 0, 1)
            imgs.append(np.ascontiguousarray(img)[np.newaxis,:])
        # batch
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0).astype(dtype = np.float32))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()


import os
import magicmind.python.runtime as mm

from logger import Logger

log = Logger()


class MMRunner:
    """Wrapper for detector's inference with MagicMindRuntime."""

    def __init__(self, mm_file, device_id):
        super(MMRunner, self).__init__()
        # check mm_file
        assert os.path.exists(mm_file), "mm model file not exist,please check it."
        assert device_id >= 0, "invalid device_id,please check it."

        self.model = mm.Model()
        self.model.deserialize_from_file(mm_file)
        log.info("Model instance Created Success!")

        with mm.System() as mm_sys:
            dev_count = mm_sys.device_count()
            assert dev_count > 0, "dev_count must be large than 0"

            # check device_id
            assert device_id < dev_count, "deviced_id must be less than dev_count"

            self.dev = mm.Device()
            self.dev.id = device_id
            assert self.dev.active().ok()
            log.info("Model dev Created Success!")

            self.econfig = mm.Model.EngineConfig()
            self.econfig.device_type = "MLU"

            self.engine = self.model.create_i_engine(self.econfig)
            assert self.engine != None, "Failed to create engine"
            log.info("Model engine Created Success!")

            self.context = self.engine.create_i_context()
            assert self.context != None, "Failed to create context"
            log.info("Model context Created Success!")

            self.queue = self.dev.create_queue()
            assert self.queue != None, "Failed to create queue"
            log.info("Model queue Created Success!")

            self.inputs = self.context.create_inputs()
            log.info("Model inputs Created Success!")
            log.info("All Model resource Created Success!")

            self.input_nums = self.model.get_input_num()
            self.outputs = []

    def __del__(self):
        _all_attrs = dir(self)

        if "inputs" in _all_attrs:
            for t in self.inputs:
                del t
            log.info("Model inputs Destoryed Success!")

        if "context" in _all_attrs:
            del self.context
            log.info("Model context Destoryed Success!")

        if "engine" in _all_attrs:
            del self.engine

        if "model" in _all_attrs:
            del self.model
            log.info("Model instance Destoryed Success!")

        if "queue" in _all_attrs:
            del self.queue
            log.info("Model queue Destoryed Success!")

        if "dev" in _all_attrs:
            del self.dev
            log.info("Model dev Destoryed Success!")

        log.info("All Model resource Destoryed Success!")

    def __call__(self, inputs, **kwargs):
        # check inputs
        assert isinstance(
            inputs, list
        ), "inputs data type must be list,please check it."

        # model infer
        for _input_idx in range(self.input_nums):
            self.inputs[_input_idx].from_numpy(inputs[_input_idx])
            self.inputs[_input_idx].to(self.dev)

        self.outputs = []

        assert self.context.enqueue(self.inputs, self.outputs, self.queue).ok()
        assert self.queue.sync().ok()

        # get output
        outputs = []
        for _output_idx in range(self.model.get_output_num()):
            outputs.append(self.outputs[_output_idx].asnumpy())

        return outputs



# Copyright (c) 2021 Mobvoi Inc. (authors: Binbin Zhang)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random

import torch
import torch.distributed as dist
from torch.utils.data import IterableDataset

import wenet.dataset.processor as processor
from wenet.utils.file_utils import read_lists


class Processor(IterableDataset):
    def __init__(self, source, f, *args, **kw):
        assert callable(f)
        self.source = source
        self.f = f
        self.args = args
        self.kw = kw

    def set_epoch(self, epoch):
        self.source.set_epoch(epoch)

    def __iter__(self):
        """ Return an iterator over the source dataset processed by the
            given processor.
        """
        assert self.source is not None
        assert callable(self.f)
        return self.f(iter(self.source), *self.args, **self.kw)

    def apply(self, f):
        assert callable(f)
        return Processor(self, f, *self.args, **self.kw)


class DistributedSampler:
    def __init__(self, shuffle=True, partition=True):
        self.epoch = -1
        self.update()
        self.shuffle = shuffle
        self.partition = partition

    def update(self):
        assert dist.is_available()
        if dist.is_initialized():
            self.rank = dist.get_rank()
            self.world_size = dist.get_world_size()
        else:
            self.rank = 0
            self.world_size = 1
        worker_info = torch.utils.data.get_worker_info()
        if worker_info is None:
            self.worker_id = 0
            self.num_workers = 1
        else:
            self.worker_id = worker_info.id
            self.num_workers = worker_info.num_workers
        return dict(rank=self.rank,
                    world_size=self.world_size,
                    worker_id=self.worker_id,
                    num_workers=self.num_workers)

    def set_epoch(self, epoch):
        self.epoch = epoch

    def sample(self, data):
        """ Sample data according to rank/world_size/num_workers

            Args:
                data(List): input data list

            Returns:
                List: data list after sample
        """
        data = list(range(len(data)))
        # TODO(Binbin Zhang): fix this
        # We can not handle uneven data for CV on DDP, so we don't
        # sample data by rank, that means every GPU gets the same
        # and all the CV data
        if self.partition:
            if self.shuffle:
                random.Random(self.epoch).shuffle(data)
            data = data[self.rank::self.world_size]
        data = data[self.worker_id::self.num_workers]
        return data


class DataList(IterableDataset):
    def __init__(self, lists, shuffle=True, partition=True):
        self.lists = lists
        self.sampler = DistributedSampler(shuffle, partition)

    def set_epoch(self, epoch):
        self.sampler.set_epoch(epoch)

    def __iter__(self):
        sampler_info = self.sampler.update()
        indexes = self.sampler.sample(self.lists)
        for index in indexes:
            # yield dict(src=src)
            data = dict(src=self.lists[index])
            data.update(sampler_info)
            yield data


def Dataset(data_type,
            data_list_file,
            symbol_table,
            conf,
            bpe_model=None,
            non_lang_syms=None,
            partition=True):
    """ Construct dataset from arguments

        We have two shuffle stage in the Dataset. The first is global
        shuffle at shards tar/raw file level. The second is global shuffle
        at training samples level.

        Args:
            data_type(str): raw/shard
            bpe_model(str): model for english bpe part
            partition(bool): whether to do data partition in terms of rank
    """
    assert data_type in ['raw', 'shard']
    lists = read_lists(data_list_file)
    shuffle = conf.get('shuffle', True)
    dataset = DataList(lists, shuffle=shuffle, partition=partition)
    if data_type == 'shard':
        dataset = Processor(dataset, processor.url_opener)
        dataset = Processor(dataset, processor.tar_file_and_group)
    else:
        dataset = Processor(dataset, processor.parse_raw)

    dataset = Processor(dataset, processor.tokenize, symbol_table, bpe_model,
                        non_lang_syms, conf.get('split_with_space', False))
    filter_conf = conf.get('filter_conf', {})
    dataset = Processor(dataset, processor.filter, **filter_conf)

    resample_conf = conf.get('resample_conf', {})
    dataset = Processor(dataset, processor.resample, **resample_conf)

    speed_perturb = conf.get('speed_perturb', False)
    if speed_perturb:
        dataset = Processor(dataset, processor.speed_perturb)

    feats_type = conf.get('feats_type', 'fbank')
    assert feats_type in ['fbank', 'mfcc']
    if feats_type == 'fbank':
        fbank_conf = conf.get('fbank_conf', {})
        dataset = Processor(dataset, processor.compute_fbank, **fbank_conf)
    elif feats_type == 'mfcc':
        mfcc_conf = conf.get('mfcc_conf', {})
        dataset = Processor(dataset, processor.compute_mfcc, **mfcc_conf)

    spec_aug = conf.get('spec_aug', True)
    spec_sub = conf.get('spec_sub', False)
    if spec_aug:
        spec_aug_conf = conf.get('spec_aug_conf', {})
        dataset = Processor(dataset, processor.spec_aug, **spec_aug_conf)
    if spec_sub:
        spec_sub_conf = conf.get('spec_sub_conf', {})
        dataset = Processor(dataset, processor.spec_sub, **spec_sub_conf)

    if shuffle:
        shuffle_conf = conf.get('shuffle_conf', {})
        dataset = Processor(dataset, processor.shuffle, **shuffle_conf)

    sort = conf.get('sort', True)
    if sort:
        sort_conf = conf.get('sort_conf', {})
        dataset = Processor(dataset, processor.sort, **sort_conf)

    batch_conf = conf.get('batch_conf', {})
    dataset = Processor(dataset, processor.batch, **batch_conf)
    dataset = Processor(dataset, processor.padding)
    return dataset



import shutil
from batchgenerators.utilities.file_and_folder_operations import *
from typing import Tuple, List
import numpy as np
import torch
import torch.nn.functional as F
from nnunet.preprocessing.preprocessing import PreprocessorFor2D
from copy import deepcopy


def check_input_folder_and_return_caseIDs(input_folder, expected_num_modalities):
    print("This model expects %d input modalities for each image" % expected_num_modalities)
    files = subfiles(input_folder, suffix=".nii.gz", join=False, sort=True)

    maybe_case_ids = np.unique([i[:-12] for i in files])

    remaining = deepcopy(files)
    missing = []

    assert len(files) > 0, "input folder did not contain any images (expected to find .nii.gz file endings)"

    # now check if all required files are present and that no unexpected files are remaining
    for c in maybe_case_ids:
        for n in range(expected_num_modalities):
            expected_output_file = c + "_%04.0d.nii.gz" % n
            if not isfile(join(input_folder, expected_output_file)):
                missing.append(expected_output_file)
            else:
                remaining.remove(expected_output_file)

    print("Found %d unique case ids, here are some examples:" % len(maybe_case_ids),
          np.random.choice(maybe_case_ids, min(len(maybe_case_ids), 10)))
    print("If they don't look right, make sure to double check your filenames. They must end with _0000.nii.gz etc")

    if len(remaining) > 0:
        print("found %d unexpected remaining files in the folder. Here are some examples:" % len(remaining),
              np.random.choice(remaining, min(len(remaining), 10)))

    if len(missing) > 0:
        print("Some files are missing:")
        print(missing)
        raise RuntimeError("missing files in input_folder")

    return maybe_case_ids

def compute_steps_for_sliding_window(patch_size: Tuple[int, ...], image_size: Tuple[int, ...], step_size: float) -> List[List[int]]:
    import numpy as np
    assert [i >= j for i, j in zip(image_size, patch_size)], "image size must be as large or larger than patch_size"
    assert 0 < step_size <= 1, 'step_size must be larger than 0 and smaller or equal to 1'

    # our step width is patch_size*step_size at most, but can be narrower. For example if we have image size of
    # 110, patch size of 32 and step_size of 0.5, then we want to make 4 steps starting at coordinate 0, 27, 55, 78
    target_step_sizes_in_voxels = [i * step_size for i in patch_size]

    num_steps = [int(np.ceil((i - k) / j)) + 1 for i, j, k in zip(image_size, target_step_sizes_in_voxels, patch_size)]

    steps = []
    for dim in range(len(patch_size)):
        # the highest step value for this dimension is
        max_step_value = image_size[dim] - patch_size[dim]
        if num_steps[dim] > 1:
            actual_step_size = max_step_value / (num_steps[dim] - 1)
        else:
            actual_step_size = 99999999999  # does not matter because there is only one step at 0

        steps_here = [int(np.round(actual_step_size * i)) for i in range(num_steps[dim])]

        steps.append(steps_here)

    return steps

def preprocess(args):
    info = load_pickle(join(args.model_path + "/fold_0/", "%s.model.pkl" % "model_final_checkpoint"))
    maybe_mkdir_p(args.output_folder)
    shutil.copy(join(args.model_path, 'plans.pkl'), args.output_folder)

    assert isfile(join(args.model_path,
                    "plans.pkl")), "Folder with saved model weights must contain a plans.pkl file"
    expected_num_modalities = load_pickle(join(args.model_path, "plans.pkl"))['num_modalities']

    # check input folder integrity
    case_ids = check_input_folder_and_return_caseIDs(args.data_folder, expected_num_modalities)

    output_files = [join(args.output_folder, i + ".nii.gz") for i in case_ids]
    all_files = subfiles(args.data_folder, suffix=".nii.gz", join=False, sort=True)
    list_of_lists = [[
        join(args.data_folder, i) for i in all_files
        if i[:len(j)].startswith(j) and len(i) == (len(j) + 12)
    ] for j in case_ids]

    plans = info['plans']
    normalization_schemes = plans['normalization_schemes']
    use_mask_for_norm = plans['use_mask_for_norm']
    transpose_forward = plans['transpose_forward']
    intensity_properties = plans['dataset_properties']['intensityproperties']
    transpose_backward = plans['transpose_backward']

    init = info['init']
    _, _, _, _, _, stage, _, _, _ = init
    stage_plans = plans['plans_per_stage'][stage]
    batch_size = stage_plans['batch_size']
    net_pool_per_axis = stage_plans['num_pool_per_axis']
    patch_size = np.array(stage_plans['patch_size']).astype(int)
    do_dummy_2D_aug = stage_plans['do_dummy_2D_data_aug']
    target_spacing = plans['plans_per_stage'][stage]['current_spacing']
    pad_border_mode = 'constant'
    pad_kwargs = {'constant_values': 0}
    step_size = 0.5
    num_classes = 2
    mirror_idx = 4
    softmax_helper = lambda x: F.softmax(x, 1)
    inference_apply_nonlin = softmax_helper
    num_results = 4
    mirror_axes = (0, 1)
    regions_class_order = None

    if 'segmentation_export_params' in plans.keys():
        force_separate_z = plans['segmentation_export_params']['force_separate_z']
        interpolation_order = plans['segmentation_export_params']['interpolation_order']
        interpolation_order_z = plans['segmentation_export_params']['interpolation_order_z']
    else:
        force_separate_z = None
        interpolation_order = 1
        interpolation_order_z = 0

    # TODO: extend for 3d
    preprocessor = PreprocessorFor2D(normalization_schemes, use_mask_for_norm, transpose_forward,
                                    intensity_properties)
    return preprocessor, list_of_lists, output_files, target_spacing, patch_size, inference_apply_nonlin, transpose_forward, transpose_backward, interpolation_order, force_separate_z, interpolation_order_z



 



import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import numpy as np
import glob
import os 
import argparse
import cv2
from calibrator import CalibData
import time

# import common components
from framework_parser import OnnxParser
from common_calibrate import common_calibrate
from build_param import get_argparser
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger

log = Logger()

WIDTH = 224
HEIGHT = 224
MAX_SAMPLES = 1000

def get_network(args):
    parser = OnnxParser(args)
    network = parser.parse()
    output_count = network.get_output_count()
    return network

def get_args():
    # get common argparser,here is onnx_parser
    arg_parser = get_argparser()
    arg_parser.add_argument(
        "--image_dir",
        dest="image_dir",
        type=str,
        default= "../../datasets/cifar100",
        help="image_dir",
    )

    # add custom args belonging to the current net.
    return arg_parser.parse_args()

def calibrate(args, network: mm.Network, config: mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    sample_data = []
    #dim = mm.Dims((args.batch_size[0], 3, HEIGHT, WIDTH))
    #data1 = CalibData(dim,0, MAX_SAMPLES)
    sample_data.append(CalibData(mm.Dims((args.batch_size[0], 3, HEIGHT, WIDTH)), 0, MAX_SAMPLES))
    sample_data.append(CalibData(mm.Dims((args.batch_size[0], 3, HEIGHT, WIDTH)), 1, MAX_SAMPLES))
    #common_calibrate(args, network, config, sample_data)
    calibrator = mm.Calibrator(sample_data)
    #assert calibrator is not None
    # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQM_ALGORITHM）。
    assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
    assert calibrator.calibrate(network, config).ok()


def main():
    # get net args
    begin_time = time.time()

    args = get_args()
    
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)
    
    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)

    log.info("Build model...")
    # 生成模型并导出
    model_name = "network"
    #build_and_serialize_params = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))

if __name__ == "__main__":
    main()




import torch
import transformers 
from transformers.data.processors.squad import SquadResult

def postprocess(features, example_indices, outputs):
    results = []
    for i, feature_index in enumerate(example_indices):
        eval_feature = features[feature_index.item()]
        unique_id = int(eval_feature.unique_id)
        output = [output[i].tolist() for output in outputs]
        start_logits, end_logits = output
        result = SquadResult(unique_id, start_logits, end_logits)
        results.append(result)
    return results


import logging
import numpy as np
import torch
from PIL import Image
from functools import lru_cache
from functools import partial
from itertools import repeat
from multiprocessing import Pool
from os import listdir
from os.path import splitext, isfile, join
from pathlib import Path
from torch.utils.data import Dataset
from tqdm import tqdm


def load_image(filename):
    ext = splitext(filename)[1]
    if ext == '.npy':
        return Image.fromarray(np.load(filename))
    elif ext in ['.pt', '.pth']:
        return Image.fromarray(torch.load(filename).numpy())
    else:
        return Image.open(filename)


def unique_mask_values(idx, mask_dir, mask_suffix):
    mask_file = list(mask_dir.glob(idx + mask_suffix + '.*'))[0]
    mask = np.asarray(load_image(mask_file))
    if mask.ndim == 2:
        return np.unique(mask)
    elif mask.ndim == 3:
        mask = mask.reshape(-1, mask.shape[-1])
        return np.unique(mask, axis=0)
    else:
        raise ValueError(f'Loaded masks should have 2 or 3 dimensions, found {mask.ndim}')


class BasicDataset(Dataset):
    def __init__(self, images_dir: str, mask_dir: str, scale: float = 1.0, mask_suffix: str = ''):
        self.images_dir = Path(images_dir)
        self.mask_dir = Path(mask_dir)
        assert 0 < scale <= 1, 'Scale must be between 0 and 1'
        self.scale = scale
        self.mask_suffix = mask_suffix

        self.ids = [splitext(file)[0] for file in listdir(images_dir) if isfile(join(images_dir, file)) and not file.startswith('.')]
        if not self.ids:
            raise RuntimeError(f'No input file found in {images_dir}, make sure you put your images there')

        logging.info(f'Creating dataset with {len(self.ids)} examples')
        logging.info('Scanning mask files to determine unique values')
        with Pool() as p:
            unique = list(tqdm(
                p.imap(partial(unique_mask_values, mask_dir=self.mask_dir, mask_suffix=self.mask_suffix), self.ids),
                total=len(self.ids)
            ))

        self.mask_values = list(sorted(np.unique(np.concatenate(unique), axis=0).tolist()))
        logging.info(f'Unique mask values: {self.mask_values}')

    def __len__(self):
        return len(self.ids)

    @staticmethod
    def preprocess(mask_values, pil_img, scale, is_mask):
        w, h = pil_img.size
        newW, newH = int(scale * w), int(scale * h)
        assert newW > 0 and newH > 0, 'Scale is too small, resized images would have no pixel'
        pil_img = pil_img.resize((newW, newH), resample=Image.NEAREST if is_mask else Image.BICUBIC)
        img = np.asarray(pil_img)

        if is_mask:
            mask = np.zeros((newH, newW), dtype=np.int64)
            for i, v in enumerate(mask_values):
                if img.ndim == 2:
                    mask[img == v] = i
                else:
                    mask[(img == v).all(-1)] = i

            return mask

        else:
            img = img.astype(np.float32)
            if img.ndim == 2:
                img = img[np.newaxis, ...]
            else:
                img = img.transpose((2, 0, 1))

            #if (img > 1).any():
            #    img = img / 255.0

            return img

    def __getitem__(self, idx):
        name = self.ids[idx]
        mask_file = list(self.mask_dir.glob(name + self.mask_suffix + '.*'))
        img_file = list(self.images_dir.glob(name + '.*'))

        assert len(img_file) == 1, f'Either no image or multiple images found for the ID {name}: {img_file}'
        assert len(mask_file) == 1, f'Either no mask or multiple masks found for the ID {name}: {mask_file}'
        mask = load_image(mask_file[0])
        img = load_image(img_file[0])

        assert img.size == mask.size, \
            f'Image and mask {name} should be the same size, but are {img.size} and {mask.size}'

        img = self.preprocess(self.mask_values, img, self.scale, is_mask=False)
        mask = self.preprocess(self.mask_values, mask, self.scale, is_mask=True)

        return {
            'image': torch.as_tensor(img.copy()).float().contiguous(),
            'mask': torch.as_tensor(mask.copy()).long().contiguous()
        }


class CarvanaDataset(BasicDataset):
    def __init__(self, images_dir, mask_dir, scale=1):
        super().__init__(images_dir, mask_dir, scale, mask_suffix='_mask')



import magicmind.python.runtime as mm
import numpy as np
import glob
import os 
import cv2
from PIL import Image
import torchvision
import torchvision.transforms.functional as F
import random

def resize(image, target, size, max_size=None, image_set='train'):
    def get_size_with_aspect_ratio(image_size, size, max_size=None):
        w, h = image_size
        if max_size is not None:
            min_original_size = float(min((w, h)))
            max_original_size = float(max((w, h)))
            if max_original_size / min_original_size * size > max_size:
                size = int(round(max_size * min_original_size / max_original_size))
        if (w <= h and w == size) or (h <= w and h == size):
            return h, w
        if w < h:
            ow = size
            oh = int(size * h / w)
        else:
            oh = size
            ow = int(size * w / h)
        return oh, ow

    rescale_size = get_size_with_aspect_ratio(image_size=image.size, size=size, max_size=max_size)
    rescaled_image = F.resize(image, rescale_size)

    if target is None:
        return rescaled_image, None
    target = target.copy()
    if image_set in ['test']:
        return rescaled_image, target

    return rescaled_image, target


class ToTensor(object):
    def __call__(self, img, target, image_set='train'):
        return torchvision.transforms.functional.to_tensor(img), target


class Normalize(object):
    def __init__(self, mean, std):
        self.mean = mean
        self.std = std

    def __call__(self, image, target, image_set='train'):
        image = torchvision.transforms.functional.normalize(image, mean=self.mean, std=self.std)
        if target is None:
            return image, None
        target = target.copy()
        if image_set in ['test']:
            return image, target
        h, w = image.shape[-2:]
        return image, target



class RandomResize(object):
    def __init__(self, sizes, max_size=None):
        assert isinstance(sizes, (list, tuple))
        self.sizes = sizes
        self.max_size = max_size

    def __call__(self, img, target=None, image_set='train'):
        size = random.choice(self.sizes)
        return resize(img, target, size, self.max_size, image_set)



class Compose(object):
    def __init__(self, transforms):
        self.transforms = transforms

    def __call__(self, image, target, image_set='train'):
        for t in self.transforms:
            image, target = t(image, target, image_set)
        return image, target

def make_hico_transforms(image_set, test_scale=-1):
    scales = [480, 512, 544, 576, 608, 640, 672, 704, 736, 768, 800]
    normalize = Compose([
        ToTensor(),
        Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225]),
    ])

    if image_set in ['test']:
        if test_scale == -1:
            return Compose([
                normalize,
            ])
        assert 400 <= test_scale <= 800*2, test_scale
        return Compose([
            RandomResize([test_scale], max_size=1333*2),
            normalize,
        ])
    raise ValueError(f'unknown {image_set}')



class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.png')
        #t = glob.glob(img_dir + '/*.JPEG')
        #self.data_paths_ += t
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        image_set = "test"
        test_scale = 672
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i], cv2.IMREAD_COLOR)
            
            img = Image.fromarray(img[:, :, ::-1]).convert('RGB')
            transforms = make_hico_transforms(image_set, test_scale)
            target = None
            img, target = transforms(img, target, image_set)
            img = img.numpy()
            imgs.append(np.ascontiguousarray(img)[np.newaxis,:])
        # batch
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import numpy as np
import cv2
from typing import List
import os
import glob
import math
import magicmind.python.runtime as mm
def preprocess_image(img, dst_shape) -> np.ndarray:
    # resize
    img = cv2.resize(img, (dst_shape[1], dst_shape[0]), interpolation=cv2.INTER_LINEAR)
    # BGR to RGB
    img = img[:, :, ::-1]
    # normalize
    MEAN = [0.40789654 * 255, 0.44719302 * 255, 0.47026115 * 255]
    STD = [0.28863828 * 255, 0.27408164 * 255, 0.27809835 * 255]
    img = (img - MEAN) / STD
    # HWC to CHW
    img = img.astype(dtype = np.float32).transpose(2, 0, 1)
    return img

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(2), self.shape_.GetDimValue(3))

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            img = preprocess_image(img, self.dst_shape_)
            imgs.append(img[np.newaxis,:])
        # batch and normalize
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



# -*- coding: utf-8 -*-
from magicmind.python.runtime.parser import Parser
import magicmind.python.runtime as mm
import argparse

def do_calibrate(network, calib_data, config, precision):
    calibrator = mm.Calibrator([calib_data])
    assert calibrator is not None

    assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
    assert config.parse_from_string(
        """{"precision_config": {"precision_mode": "%s"}}"""%(precision)).ok()
    assert calibrator.calibrate(network, config).ok()

def construct_model(args, reload_by_serilize=True):
    # init builder, network, builder_config and parser
    builder = mm.Builder()
    network = mm.Network()
    config = mm.BuilderConfig()
    parser = Parser(mm.ModelKind.kTensorflow)

    # get input dims from network
    parser.set_model_param("tf-model-type", "tf-graphdef-file")
    parser.set_model_param("tf-graphdef-inputs", ["input_1"])
    parser.set_model_param("tf-graphdef-outputs", ["pred_pose/mul_24"])
    
    assert parser.parse(network, args.tf_pb).ok()
    assert network.get_input(0).set_dimension(mm.Dims((args.batch_size, args.img_size[0], args.img_size[1], 3))).ok()
    config.parse_from_string('{"archs":[{"mtp_372": [2,6,8]}]}')
    config.parse_from_string('{"opt_config":{"type64to32_conversion": true}}')
    config.parse_from_string('{"opt_config":{"conv_scale_fold": true}}')
    if args.shape_mutable=='true':
        assert config.parse_from_string('{"graph_shape_mutable": true}').ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable": false}').ok()  

    model = builder.build_model("mm_model", network, config)
    assert model.serialize_to_file(args.mm_model).ok()
    print(args.mm_model," was saved successfully in the current path.")

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--tf_pb', type=str, default='../data/models/psenet.pb', help='tf_pb path(s)')
    parser.add_argument('--mm_model', type=str, default='../data/', help='saved .mm model name')
    parser.add_argument('--precision', type=str, default='force_float32', help='precision')
    parser.add_argument('--shape_mutable', type=str, default="false", required=True ,help='shape_mutable')
    parser.add_argument('--img_size', type=list, default=[64, 64], help='inference size (pixels)')
    parser.add_argument('--batch_size', type=int, default=1, help='inference batch, default:1') 
    args = parser.parse_args()    

    if args.precision != "force_float32":
        print("FSA-Net Only Support FP32(force_float32) Mode !")
        exit(0)
    construct_model(args)




from __future__ import print_function
from __future__ import division

import argparse
import random
import torch
#import torch.backends.cudnn as cudnn
import torch.optim as optim
import torch.utils.data
from torch.autograd import Variable
import numpy as np
import os
import dataset
import util
import magicmind.python.runtime as mm
from PIL import Image
import linecache
import sys
#sys.path.append("../../../utils")
from utils import Record
cur_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.append(cur_dir + "/../export_model/crnn.pytorch")
import models.crnn as crnn

from mm_runner import MMRunner
from logger import Logger

log = Logger()


def get_line_context(file_path, line_number):
    return linecache.getline(file_path, line_number).strip()
parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
parser.add_argument("--valRoot", required=True, help="path to dataset")
parser.add_argument("--file_path", required=True, help="path to lexicon.txt")
parser.add_argument("--magicmind_model", required=True, help="path to mm model")
parser.add_argument(
    "--workers", type=int, help="number of data loading workers", default=2
)
parser.add_argument("--batch_size", type=int, default=64, help="input batch size")
parser.add_argument(
    "--imgH", type=int, default=32, help="the height of the input image to network"
)
parser.add_argument(
    "--imgW", type=int, default=100, help="the width of the input image to network"
)
parser.add_argument("--nh", type=int, default=256, help="size of the lstm hidden state")
parser.add_argument(
    "--alphabet", type=str, default="0123456789abcdefghijklmnopqrstuvwxyz"
)
parser.add_argument(
    "--n_test_disp", type=int, default=10, help="Number of samples to display when test"
)
parser.add_argument("--manualSeed", type=int, default=1234, help="reproduce experiemnt")

parser.add_argument(
    "--image_num", dest="image_num", type=int, default=1000, help="image number"
)

parser.add_argument(
    "--top1_file",
    type=str,
    default="../data/output/result.txt",
    help="path to save result",
)

opt = parser.parse_args()
print(opt)

random.seed(opt.manualSeed)
np.random.seed(opt.manualSeed)
torch.manual_seed(opt.manualSeed)

#cudnn.benchmark = True

test_dataset = dataset.lmdbDataset(
    root=opt.valRoot, transform=dataset.resizeNormalize((100, 32))
    )

print("WARNING: warpctc is not supported !")
from torch.nn import CTCLoss
#cudnn.enabled = False
criterion = torch.nn.CTCLoss(reduction="sum", zero_infinity=True)

image_tmp = torch.FloatTensor(opt.batch_size, 3, opt.imgH, opt.imgH)
text = torch.IntTensor(opt.batch_size * 5)
length = torch.IntTensor(opt.batch_size)

image_tmp = Variable(image_tmp)
text = Variable(text)
length = Variable(length)

print("Start infer")
model_path = opt.magicmind_model
model = MMRunner(mm_file=opt.magicmind_model, device_id=opt.device_id)

converter = util.strLabelConverter(opt.alphabet)
data_loader = torch.utils.data.DataLoader(
    test_dataset, shuffle=True, batch_size=opt.batch_size, num_workers=int(opt.workers)
    )
val_iter = iter(data_loader)
epoch_size = len(data_loader)

n_correct = 0
loss_avg = util.averager()
print("epoch_size:", epoch_size)

IMAGE_HEIGHT=opt.imgH
IMAGE_WIDTH=opt.imgW

image_num = opt.image_num
batch_size = opt.batch_size
max_iter = min(opt.image_num, epoch_size)
rem_img_num = image_num % batch_size
img_idx = 0
batch_counter = 0

for i in range(max_iter):
    #data = val_iter.next()
    data = next(iter(val_iter))
    cpu_images, cpu_texts = data
    #batch_size = cpu_images.size(0)
    infer_batch_size = (
            batch_size if img_idx < (image_num - rem_img_num) else rem_img_num
        )
    if infer_batch_size == 0:
        break

    util.loadData(image_tmp, cpu_images)
    t, l = converter.encode(cpu_texts)
    util.loadData(text, t)
    util.loadData(length, l)
    outputs = []
    image_vec = torch.split(image_tmp,1,dim=0)
    imgs = np.empty([infer_batch_size, 1, IMAGE_HEIGHT, IMAGE_WIDTH])
    #for index in range (0,infer_batch_size):
    image = image_vec[i%infer_batch_size]
    imgs[batch_counter % infer_batch_size, :, :, :] = image
    batch_counter += 1
    img_idx += 1
    if batch_counter % infer_batch_size == 0:
        batch_counter = 0
        inputs = [imgs]
        outputs = model(inputs)
        for pred_idx in range(infer_batch_size):
            preds= torch.tensor(outputs[0].astype(np.float32))
            preds_size = Variable(torch.IntTensor([preds.size(0)]*infer_batch_size))
            _, preds = preds.max(2)
            preds = preds.transpose(1, 0).contiguous().view(-1)
            sim_preds = converter.decode(preds.data, preds_size.data, raw=False)
            if batch_size == 1:
                sim_preds = [sim_preds]

            for pred, target in zip(sim_preds, cpu_texts):
                target = get_line_context(opt.file_path, int(target) + 1)
            if pred == target.lower():
                n_correct += 1

raw_preds = converter.decode(preds.data, preds_size.data, raw=True)[: opt.n_test_disp]
if batch_size == 1:
    raw_preds = [raw_preds]
for raw_pred, pred, gt in zip(raw_preds, sim_preds, cpu_texts):
    print(
            "%-20s => %-20s, gt: %-20s"
            % (raw_pred, pred, get_line_context(opt.file_path, int(gt)+1)))

accuracy = n_correct / float(max_iter)
result_top1 = Record(opt.top1_file)
result_top1.write("top1 accuracy: %f" % (accuracy), True)



import sys
sys.path.append('CenterNet-2b7692c377c6686fb35e473dac2de6105eed62c6/src/')
import argparse
import torch
import _init_paths
from models.networks.dlav0 import DLASeg as CenterNet
from models.model import load_model
parser = argparse.ArgumentParser()
parser.add_argument("--model_weight", dest = 'model_weight', type=str,default="../data/models/")
parser.add_argument('--input_width', dest = 'input_width', default = 512, type = int, help = 'model input width')
parser.add_argument('--input_height', dest = 'input_height', default = 512, type = int, help = 'model input height')
parser.add_argument('--batch_size', dest = 'batch_size', default = 1, type = int, help = 'model input batch')
parser.add_argument("--traced_pt", dest = 'traced_pt', type=str, default = "../data/models/")

args = parser.parse_args()
# 载入权重
heads = {'hm' : 80, 'wh' : 2, 'reg' :2}
model = CenterNet('dla34', heads, pretrained=True, down_ratio=4, head_conv=256)
model = load_model(model, args.model_weight)
model.eval()

# jit.trace
traced_model = torch.jit.trace(model, torch.rand(args.batch_size, 3, args.input_height, args.input_width))
traced_model.save(args.traced_pt)



import magicmind.python.runtime as mm

from logger import Logger

log = Logger()


def common_calibrate(args, network, config, calib_data):
    # network : mm.Network
    # config : mm.BuilderConfig
    # calib_data: instance of class CalibData
    # 创建量化工具并设置量化统计算法
    # calib_data = CalibData(shape = input_dims, max_samples = args.max_sample, img_dir = args.image_dir)
    calibrator = mm.Calibrator([calib_data])
    assert calibrator is not None
    # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQNM_ALGORITHM）。
    assert args.calibration_algo is not None, "calibration algorithm is not specified."
    PARAM_QUANT_ALGO_2_MM = {
        "linear": mm.QuantizationAlgorithm.LINEAR_ALGORITHM,
        "eqnm": mm.QuantizationAlgorithm.EQNM_ALGORITHM,
    }
    assert calibrator.set_quantization_algorithm(
        PARAM_QUANT_ALGO_2_MM[args.calibration_algo]
    ).ok()
    log.info("calibration: set quantization algotithm successfully.")
    # 打开设备
    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        log.info("Device count:{}".format(dev_count))
        if args.device_id >= dev_count:
            log.info("Invalid device set!")
            abort()
        # 打开MLU设备
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
    # 进行量化
    assert calibrator.calibrate(network, config).ok()



import magicmind.python.runtime as mm
import numpy as np

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, index: int, max_samples: int, input_data: list):
        super().__init__()
        self.shape_ = shape
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.max_samples_ = max_samples
        self.index = index
        self.input_data = input_data

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        if self.index < 26:
            return mm.DataType.INT32
        else:
            return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess(self, input_data: list) -> np.ndarray:
    
        if self.index < 26:
            input_data[self.index].astype(np.int32)
        return input_data[self.index]

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess(self.input_data)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()